/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antminer;

import java.util.Collection;

import net.sf.myra.framework.Colony;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.Pheromone;
import net.sf.myra.framework.PheromoneUpdater;
import net.sf.myra.framework.Trail;
import net.sf.myra.framework.Vertex;

/**
 * This class represents a pheromone update procedure which initializes both
 * pheromone and heuristic values.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class DefaultPheromoneUpdater
	implements PheromoneUpdater
{
	public void initialize(Colony colony)
	{
		Graph<?> graph = colony.getTrailFactory().getGraph();

		// initial pheromone value for all vertices
		Pheromone pheromone =
			new Pheromone(1.0 / (double) (graph.getVertices().size()));

		for (Vertex<?> vertex : graph.getVertices())
		{
			vertex.setPheromone(pheromone);
		}
	}

	public void evaporate(Colony colony)
	{
		double total = 0.0;

		for (Vertex<?> vertex : colony.getTrailFactory().getGraph().vertices())
		{
			total += vertex.getPheromone().getValue();
		}

		// normalize the pheromone values, indirectly decreasing the pheromone
		// value of unused terms (vertices)
		for (Vertex<?> vertex : colony.getTrailFactory().getGraph().vertices())
		{
			double value = vertex.getPheromone().getValue();
			vertex.getPheromone().setValue(value / total);
		}
	}

	public void update(Trail trail, Colony colony)
	{
		Collection<Vertex<?>> used = trail.getVertices();

		// updates the pheromone value of the used terms (vertices)
		for (Vertex<?> vertex : colony.getTrailFactory().getGraph().vertices())
		{
			if (used.contains(vertex))
			{
				double value = vertex.getPheromone().getValue();

				vertex.getPheromone().setValue(
					value + (value * trail.getCost().getValue()));
			}
		}
	}
}