/*
 * $Id: Ant.java 2336 2011-08-09 16:39:15Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import net.sf.myra.antree.representation.Tree;
import net.sf.myra.datamining.data.Dataset;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2336 $ $Date:: 2011-08-09 17:39:15#$
 */
public class Ant
{
	private static final TreeComparator COMPARATOR = new TreeComparator();

	/**
	 * The tree builder used by the ants.
	 */
	private static Stepper stepper;

	/**
	 * The current tree created by the ant.
	 */
	private Tree tree;

	/**
	 * The best tree created by the ant.
	 */
	private Tree memory;

	/**
	 * Creates a new tree.
	 */
	public void walk(Dataset dataset)
	{
		tree = stepper.step(dataset);

		// if the tree just built is better than any previous one, we update
		// the ant memory
		if (memory == null || COMPARATOR.compare(tree, memory) > 0)
		{
			memory = tree;
		}
	}

	/**
	 * Returns the current created tree.
	 * 
	 * @return the current created tree.
	 */
	public Tree getTree()
	{
		return tree;
	}

	/**
	 * Returns the best created tree.
	 * 
	 * @return the best created tree.
	 */
	public Tree getBestTree()
	{
		return memory;
	}

	static void setStepper(Stepper stepper)
	{
		Ant.stepper = stepper;
	}
}