/*
 * $Id: GainRatioHeuristic.java 2343 2011-08-29 11:44:38Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================ 
 */

package net.sf.myra.antree;

import java.util.HashMap;
import java.util.Map;

import net.sf.myra.antree.representation.Branch;
import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.Dataset;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2343 $ $Date:: 2011-08-29 12:44:38#$
 */
public class GainRatioHeuristic
	implements Heuristic
{
	/**
	 * The entropy helper instance.
	 */
	private EntropyHelper helper;

	/**
	 * The cached value for each predictor attribute.
	 */
	private Map<String, Double> cache = new HashMap<String, Double>();

	/**
	 * Default constructor.
	 * 
	 * @param dataset the data set instance.
	 */
	public GainRatioHeuristic(Dataset dataset)
	{
		helper = new EntropyHelper(dataset);

		for (Attribute attribute : dataset.getMetadata().getPredictor())
		{
			cache.put(attribute.getName(), helper.gainRatio(attribute));
		}
	}

	public double valueFor(Branch branch, Attribute attribute)
	{
		return cache.get(attribute.getName());
	}
}