/*
 * $Id: LeafAdjustedPruner.java 2329 2011-08-02 08:57:44Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import net.sf.myra.antree.representation.Tree;
import net.sf.myra.datamining.data.Dataset;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2329 $ $Date:: 2011-08-02 09:57:44#$
 */
public class LeafAdjustedPruner
	implements Pruner
{
	/**
	 * The leaf pruner to be applied before the user-defined pruner.
	 */
	private AccuracyPruner accuracyPruner;

	/**
	 * The user-defined pruner.
	 */
	private ErrorBasedPruner errorPruner;

	/**
	 * Default constructor.
	 */
	public LeafAdjustedPruner()
	{
		// the first-stage of the pruning maximises the predictive accuracy of 
		// the tree using the accuracy pruning
		accuracyPruner = new AccuracyPruner();

		// the second-stage of the pruning maximises the generalisation of the
		// tree using the reduced error pruning
		errorPruner = new ErrorBasedPruner();
	}

	public Tree prune(Tree tree, Dataset dataset)
	{
		return errorPruner.prune(accuracyPruner.prune(tree, dataset), dataset);
	}
}