/*
 * $Id: MaxMinStagnationTest.java 2329 2011-08-02 08:57:44Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.logging.Logger;

import net.sf.myra.antree.Graph.Vertex;
import net.sf.myra.antree.representation.Branch;
import net.sf.myra.antree.representation.InternalNode;
import net.sf.myra.antree.representation.Tree;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2329 $ $Date:: 2011-08-02 09:57:44#$
 */
public class MaxMinStagnationTest
	implements ConvergenceTest
{
	/**
	 * The logger instance.
	 */
	private Logger logger =
		Logger.getLogger(MaxMinStagnationTest.class.getName());

	public boolean hasConverged(Graph graph, Iteration iteration)
	{
		if (graph instanceof MaxMinGraph)
		{
			double tMin = ((MaxMinGraph) graph).getTMin();
			double tMax = ((MaxMinGraph) graph).getTMax();

			if (hasConverged(graph.getVertices(null), tMin, tMax))
			{
				Tree tree = iteration.getTree();
				LinkedList<Branch> branches = new LinkedList<Branch>();
				branches.addAll(Arrays.asList(tree.getRoot().getBranches()));

				while (!branches.isEmpty())
				{
					Branch branch = branches.removeFirst();

					if (branch.getTo() instanceof InternalNode
						&& !hasConverged(graph.getVertices(branch), tMin, tMax))
					{
						return false;
					}

					branches
						.addAll(Arrays.asList(branch.getTo().getBranches()));
				}

				logger.info("Search has converged at iteration "
					+ iteration.getNumber());

				return true;
			}

			return false;
		}

		throw new IllegalArgumentException("Not a max-min graph instance: "
			+ graph.getClass().getName());
	}

	private boolean hasConverged(Collection<Vertex> components, double tMin,
			double tMax)
	{
		int minCount = 0;
		int maxCount = 0;

		for (Vertex vertex : components)
		{
			double value = Util.precision(vertex.getPheromone());

			if (value == Util.precision(tMin))
			{
				minCount++;
			}

			if (value == Util.precision(tMax))
			{
				maxCount++;
			}
		}

		return ((maxCount + minCount) == components.size()) && (maxCount == 1);
	}
}