/*
 * $Id: StagnationTest.java 2320 2011-01-25 13:06:10Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import net.sf.myra.framework.Configuration;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2320 $ $Date:: 2011-01-25 13:06:10#$
 */
public class StagnationTest
	implements ConvergenceTest
{
	/**
	 * The property key under which the number of iterations to determine the
	 * stagnation of the search is stored.
	 */
	public static final String ITERATIONS = "net.sf.myra.antree.stagnation";

	/**
	 * The number of iterations to determine the stagnation of the search.
	 */
	private int iterations;

	/**
	 * The current iterations count.
	 */
	private int count;

	/**
	 * The last known different tree quality value.
	 */
	private double quality = Double.NaN;

	/**
	 * Default constructor.
	 */
	public StagnationTest()
	{
		iterations = Configuration.getIntProperty(ITERATIONS, 20);
		count = 0;
	}

	public boolean hasConverged(Graph graph, Iteration iteration)
	{
		if (!Double.isNaN(quality)
			&& (quality >= iteration.getTree().getQuality().doubleValue()))
		{
			count++;
		}
		else
		{
			quality = iteration.getTree().getQuality().doubleValue();
			count = 1;
		}

		return (count >= iterations);
	}
}