/*
 * $Id: Stepper.java 2336 2011-08-09 16:39:15Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import net.sf.myra.antree.representation.Tree;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.data.Dataset;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2336 $ $Date:: 2011-08-09 17:39:15#$
 */
public class Stepper
{
	private final Builder builder;

	private final Pruner pruner;

	private final Measure measure;

	public Stepper(Builder builder, Pruner pruner, Measure measure)
	{
		this.builder = builder;
		this.pruner = pruner;
		this.measure = measure;
	}

	public Tree step(Dataset dataset)
	{
		Tree tree = builder.build(dataset);
		tree = pruner.prune(tree, dataset);
		tree.setStatistics(tree.gather(dataset));
		tree.setQuality(measure.evaluate(dataset, tree));

		return tree;
	}
}