/*
 * $Id: ThresholdHelper.java 2336 2011-08-09 16:39:15Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import java.util.List;

import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.IntervalBuilder;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.datamining.data.IntervalBuilder.Interval;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2336 $ $Date:: 2011-08-09 17:39:15#$
 */
public class ThresholdHelper
{
	public static Term[] create(Dataset dataset, ContinuousAttribute attribute)
	{
		return create(dataset.getMetadata(), attribute, dataset.getInstances());
	}

	public static Term[] create(Metadata metadata,
			ContinuousAttribute attribute, List<Instance> instances)
	{
		if (attribute.domain().length == 0)
		{
			throw new IllegalStateException(
				"Continuous domain not calculated: " + attribute.getName());
		}

		Interval[] intervals =
			IntervalBuilder.getInstance(metadata).create(instances, attribute);

		Term[] terms = new Term[intervals.length];
		int index = metadata.getIndex(attribute.getName());

		for (int i = 0; i < terms.length; i++)
		{
			terms[i] = new Term(attribute.getName(), index);
			terms[i].setOperator(intervals[i].getOperator());
			terms[i].setValues(fix(attribute, intervals[i].getValues()));
		}

		return terms;
	}

	public static int count(Metadata metadata, ContinuousAttribute attribute,
			List<Instance> instances)
	{
		return IntervalBuilder.getInstance(metadata)
			.count(instances, attribute);
	}

	private static double[] fix(ContinuousAttribute attribute, double[] values)
	{
		double[] fixed = new double[values.length];

		for (Double v : attribute.domain())
		{
			for (int i = 0; i < values.length; i++)
			{
				if (v <= values[i] && v > fixed[i])
				{
					fixed[i] = v;
				}
			}
		}

		return fixed;
	}
}