/*
 * $Id: TreeComparator.java 2320 2011-01-25 13:06:10Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree;

import java.util.Comparator;

import net.sf.myra.antree.representation.Tree;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2320 $ $Date:: 2011-01-25 13:06:10#$
 */
public class TreeComparator
	implements Comparator<Tree>
{
	/**
	 * Compares the two specified tree. The comparison takes into account the
	 * quality and the size of the trees. A tree is considered 'bigger' if it
	 * has a higher quality value or the same quality value but it is smaller
	 * in size.
	 * 
	 * @param t1 the first tree to be compared.
	 * @param t2 the second tree to be compared.
	 * 
	 * @return <code>-1</code> if the first tree is 'smaller' that the second;
	 *         <code>0</code> if the tree are 'equal' (same quality and size);
	 *         <code>1</code> if the first tree is 'bigger' than the second
	 *         tree.
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public int compare(Tree t1, Tree t2)
	{
		int c = ((Comparable) t1.getQuality()).compareTo(t2.getQuality());

		if (c == 0)
		{
			return Double.compare(t2.size(), t1.size());
		}

		return c;
	}
}