/*
 * $Id: LeafNode.java 2320 2011-01-25 13:06:10Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree.representation;

import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2320 $ $Date:: 2011-01-25 13:06:10#$
 */
public final class LeafNode
	extends Node
{
	/**
	 * The label predicted by the leaf node.
	 */
	private Label prediction;

	/**
	 * Default constructor.
	 * 
	 * @param level the level of the node.
	 * @param metadata the current data set metadata.
	 * @param prediction the prediction of the node.
	 */
	public LeafNode(int level, Metadata metadata, Label prediction)
	{
		super(level, metadata.getTarget());
		this.prediction = prediction;
	}

	/**
	 * Returns the label predicted by the node.
	 * 
	 * @return the label predicted by the node.
	 */
	@Override
	public Label getPrediction()
	{
		return prediction;
	}

	/**
	 * Sets the prediction of the node.
	 * 
	 * @param prediction the prediction to set.
	 */
	@Override
	public void setPrediction(Label prediction)
	{
		this.prediction = prediction;
	}

	@Override
	public String label()
	{
		return prediction.toString();
	}

	@Override
	public LeafNode clone()
	{
		return (LeafNode) super.clone();
	}
}