/*
 * $Id: NodeFactory.java 2320 2011-01-25 13:06:10Z febo $
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2010 Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.antree.representation;

import java.util.List;

import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2320 $ $Date:: 2011-01-25 13:06:10#$
 */
public abstract class NodeFactory
{
	/**
	 * Creates a new node (internal or leaf) according to the type of the
	 * specified attribute.
	 * 
	 * @param attribute the attribute to be represented by the node.
	 * @param dataset the current data set.
	 * @param level the level of the node.
	 * 
	 * @return a new node (internal or leaf) according to the type of the
	 *         specified attribute.
	 * 
	 * @see InternalNode
	 * @see LeafNode
	 */
	public static Node create(Attribute attribute, Dataset dataset, int level)
	{
		return create(attribute, dataset.getMetadata(), dataset.getInstances(),
			level);
	}

	/**
	 * Creates a new node (internal or leaf) according to the type of the
	 * specified attribute.
	 * 
	 * @param attribute the attribute to be represented by the node.
	 * @param metadata the current data set metadata.
	 * @param instances the list of instances.
	 * @param level the level of the node.
	 * 
	 * @return a new node (internal or leaf) according to the type of the
	 *         specified attribute.
	 * 
	 * @see InternalNode
	 * @see LeafNode
	 */
	public static Node create(Attribute attribute, Metadata metadata,
			List<Instance> instances, int level)
	{
		if (attribute.isTarget())
		{
			return new LeafNode(level, metadata, Evaluator.findMajorityClass(
				instances, metadata));
		}
		else
		{
			return new InternalNode(level, attribute, Branch.create(metadata,
				attribute, instances));
		}
	}
}
