/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.cantminer;

import java.util.Collection;

import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Colony;
import net.sf.myra.framework.Edge;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.PheromoneUpdater;
import net.sf.myra.framework.Trail;
import net.sf.myra.framework.Vertex;

/**
 * Default cAnt-Miner2 pehromone update policy.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class EdgePheromoneUpdater
	implements PheromoneUpdater
{
	@SuppressWarnings("unchecked")
	public void evaporate(Colony colony)
	{
		Graph<Term> graph = (Graph<Term>) colony.getTrailFactory().getGraph();
		graph.apply(Graph.EDGE_EVAPORATION);
	}

	@SuppressWarnings("unchecked")
	public void initialize(Colony colony)
	{
		Graph<Term> graph = (Graph<Term>) colony.getTrailFactory().getGraph();
		graph.apply(Graph.EDGE_INITIALIZATION);
	}

	@SuppressWarnings("unchecked")
	public void update(Trail trail, Colony colony)
	{
		Collection<Vertex<?>> used = trail.getVertices();
		Graph<Term> graph = (Graph<Term>) colony.getTrailFactory().getGraph();
		Vertex<Term> from = graph.get(cAntMiner2.VIRTUAL_VERTEX);

		// updates the pheromone value of the used terms (vertices)
		for (Vertex<?> to : used)
		{
			Edge<?> edge = graph.getEdge(from, (Vertex<Term>) to);
			double value = edge.getPheromone().getValue();

			edge.getPheromone().setValue(
				value + (value * trail.getCost().getValue()));

			from = (Vertex<Term>) to;
		}
	}
}