/*
 * $Id$ 
 */

package net.sf.myra.cantminer.alpha.step;

import java.util.List;
import java.util.logging.Logger;

import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.IntervalBuilder;
import net.sf.myra.datamining.data.IntervalBuilder.Interval;
import net.sf.myra.datamining.data.Term;

/**
 * This class represents a continuous attribute step.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                    #$
 */
public final class ContinuousStep
	extends AbstractStep
{
	private static final long serialVersionUID = -4435084657693469366L;

	/**
	 * The logger instance.
	 */
	private Logger logger = Logger.getLogger(ContinuousStep.class.getName());

	/**
	 * The attribute index.
	 */
	private int index;

	/**
	 * The attribute represented by this step.
	 */
	private ContinuousAttribute attribute;

	/**
	 * The continuous attribute discretisation helper.
	 */
	private IntervalBuilder builder;

	/**
	 * Creates a new <code>ContinuousStep</code> instance.
	 * 
	 * @param attribute the attribute represented by this step.
	 * @param builder the continuous attribute discretisation helper.
	 */
	public ContinuousStep(ContinuousAttribute attribute, int index,
		IntervalBuilder builder)
	{
		this.attribute = attribute;
		this.index = index;
		this.builder = builder;
	}

	public Term select(List<Instance> instances, int level, int minimum)
	{
		Interval interval = builder.createSingle(instances, attribute);
		Term term = new Term(attribute.getName(), index);
		term.setValues(interval.getValues());
		term.setOperator(interval.getOperator());

		if (!accept(term, instances, minimum))
		{
			logger.finest("Could not create an interval for attribute: "
				+ attribute + ", number of instances: " + instances.size());
			term = null;
		}

		return term;
	}

	public ContinuousAttribute getAttribute()
	{
		return attribute;
	}
}