/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero and Matthew Medland
 */

package net.sf.myra.cantminer.pittsburgh.dynamic;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;

/** 
 * @author Fernando Esteban Barril Otero
 * @author Matthew Medland
 */
public class FunctionalRule
	extends Rule
{
	private static final long serialVersionUID = 890916959008769748L;

	/**
	 * The <code>AbstractObjectiveFunction</code> that should be used to
	 * evaluate the rule.
	 */
	private AbstractObjectiveFunction function;

	public FunctionalRule(int size, Label prototype)
	{
		super(size, prototype);
	}

	public FunctionalRule(Metadata metadata)
	{
		super(metadata);
	}

	public AbstractObjectiveFunction getFunction()
	{
		return function;
	}

	public void setFunction(AbstractObjectiveFunction function)
	{
		this.function = function;
	}

	public String getFunctionName()
	{
		return function == null ? "[none]" : function.getClass()
			.getSimpleName();
	}

	@Override
	public FunctionalRule clone()
	{
		FunctionalRule rule = (FunctionalRule) super.clone();
		// TODO if we have parametric functions with different values, we might
		// need to clone the function
		rule.setFunction(function);

		return rule;
	}
}