/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero
 */

package net.sf.myra.cantminer.pittsburgh.unordered;

import java.util.LinkedList;

import net.sf.myra.cantminer.pittsburgh.CandidateRuleList;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.model.RuleSet;
import net.sf.myra.datamining.model.RuleSet.ConflictResolutionStrategy;
import net.sf.myra.framework.Configuration;

public class UnorderedRuleList
	extends CandidateRuleList
{
	private static final long serialVersionUID = -8731637789332859909L;

	/**
	 * The property key name under which the conflict resolution strategy is
	 * stored.
	 */
	public static final String CONFLICT_STRATEGY =
		"net.sf.myra.cantminer.pittsburgh.unordered.strategy";

	/**
	 * The conflict resolution strategy based on class frequency.
	 */
	public static final String DISTRIBUTION_STRATEGY = "distribution";

	/**
	 * The conflict resolution strategy based on the rule quality.
	 */
	public static final String CONFIDENCE_STRATEGY = "confidence";

	/**
	 * The conflict resolution strategy.
	 */
	private ConflictResolutionStrategy strategy;

	/**
	 * Default constructor.
	 */
	public UnorderedRuleList()
	{
		String type =
			Configuration.getProperty(CONFLICT_STRATEGY, DISTRIBUTION_STRATEGY);

		if (type.equals(CONFIDENCE_STRATEGY))
		{
			this.strategy = RuleSet.ConflictResolutionStrategy.RULE_QUALITY;
		}
		else
		{
			this.strategy = RuleSet.ConflictResolutionStrategy.FREQUENT_CLASS;
		}
	}

	@Override
	public Label classify(Instance instance)
	{
		LinkedList<Rule> rules = new LinkedList<Rule>();

		for (Rule rule : getRules())
		{
			if (rule.isCovered(instance))
			{
				rules.add(rule);
			}
		}

		return rules.isEmpty() ? getMajority() : strategy.resolve(rules);

		/*
		HashMap<String, Integer> occurences = new HashMap<String, Integer>();

		for (Rule rule : getRules())
		{
			if (rule.isCovered(instance))
			{
				DefaultConfidenceFactor confidence =
					(DefaultConfidenceFactor) rule.getConfidence();

				for (String value : rule.getConsequent().names())
				{
					Integer frequency = occurences.get(value);

					if (frequency == null)
					{
						frequency = 0;
					}

					occurences.put(value, frequency + confidence.get(value));
				}
			}
		}

		String majority = null;
		int frequency = 0;

		for (String value : occurences.keySet())
		{
			if (occurences.get(value) > frequency)
			{
				majority = value;
				frequency = occurences.get(value);
			}
		}

		return (majority == null) ? getMajority() : instance.getLabel()
			.toLabel(majority);
		*/
	}
}