/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.cantminer.preselect;

import static net.sf.myra.cantminer.cAntMiner2.VIRTUAL_VERTEX;

import java.util.ArrayList;

import net.sf.myra.cantminer.EdgePheromoneUpdater;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Colony;
import net.sf.myra.framework.Edge;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.Trail;
import net.sf.myra.framework.Vertex;

/**
 * Default cAnt-Miner2 pehromone update policy.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ClassEdgePheromoneUpdater
	extends EdgePheromoneUpdater
{
	@SuppressWarnings("unchecked")
	public void update(Trail trail, Colony colony)
	{
		ArrayList<Vertex<?>> used =
			new ArrayList<Vertex<?>>(trail.getVertices());

		Graph<Term> graph = (Graph<Term>) colony.getTrailFactory().getGraph();
		Vertex<Term> from = graph.get(VIRTUAL_VERTEX);

		Rule rule = (Rule) trail;
		used.add(0, graph.get(rule.getConsequent().toString()));

		// updates the pheromone value of the used terms (vertices)
		for (Vertex<?> to : used)
		{
			Edge<?> edge = graph.getEdge(from, (Vertex<Term>) to);
			double value = edge.getPheromone().getValue();

			edge.getPheromone().setValue(
				value + (value * trail.getCost().getValue()));

			from = (Vertex<Term>) to;
		}
	}
}