
package net.sf.myra.cantminer.preselect;

import java.util.List;

import net.sf.myra.cantminer.ThresholdHelper;
import net.sf.myra.cantminer.pittsburgh.dynamic.FunctionTerm;
import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.NominalAttribute;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.HeuristicInformation;
import net.sf.myra.framework.HeuristicValue;
import net.sf.myra.framework.Vertex;

public class ClassHeuristicInformation
	implements HeuristicInformation
{
	/**
	 * The current data set.
	 */
	private Dataset dataset;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the current data set.
	 */
	public ClassHeuristicInformation(Dataset dataset)
	{
		this.dataset = dataset;
	}

	@SuppressWarnings("unchecked")
	public void initialize(Graph<?> graph)
	{
		Metadata metadata = dataset.getMetadata();
		NominalAttribute target = metadata.getTarget();

		for (Vertex<?> vertex : graph.getVertices())
		{
			if (!(vertex.getInfo() instanceof FunctionTerm))
			{
				if (vertex.getInfo() != null)
				{
					Term term = ((Vertex<Term>) vertex).getInfo();
					Attribute attribute = metadata.get(term.getAttribute());

					if (attribute instanceof ContinuousAttribute)
					{
						ThresholdHelper helper = new ThresholdHelper(metadata);

						for (String value : target.getValues())
						{
							Vertex<Term> v =
								helper.create((ContinuousAttribute) attribute,
									(Vertex<Term>) vertex,
									dataset.getInstances(), value);

							List<Instance> subset =
								dataset.getInstances(v.getInfo());
							double count = 0.0;

							for (Instance instance : subset)
							{
								if (instance.getLabel().active(value))
								{
									count++;
								}
							}

							vertex.setHeuristicValue(new HeuristicValue(
								(count == 0.0 ? 0.0 : count / subset.size())),
								target.getIndex(value));
						}
					}
					else
					{
						List<Instance> subset = dataset.getInstances(term);
						double[] distribution =
							new double[target.getValues().size()];

						for (Instance instance : subset)
						{
							int i =
								target.getIndex(instance.getLabel().toString());
							distribution[i]++;
						}

						for (int i = 0; i < distribution.length; i++)
						{
							vertex.setHeuristicValue(new HeuristicValue(
								(distribution[i] == 0.0 ? 0.0 : distribution[i]
									/ subset.size())), i);
						}
					}
				}
				else if (vertex.getInfo() != null)
				{
					double count =
						dataset.getInstances(vertex.getLabel()).size();

					vertex.setHeuristicValue(new HeuristicValue(
						1.0 - (count / dataset.getSize())));
				}
			}
		}
		/*
		AbstractObjectiveFunction function =
			(AbstractObjectiveFunction) ObjectFactory.create(Configuration
				.getProperty(RULE_QUALITY_FUNCTION,
					SensitivitySpecificityFunction.class.getName()),
				new Class<?>[] { Dataset.class }, new Object[] { dataset });

		Metadata metadata = dataset.getMetadata();
		NominalAttribute target = metadata.getTarget();

		for (Vertex<?> vertex : graph.getVertices())
		{
			if (vertex.getInfo() != null)
			{
				Term term = ((Vertex<Term>) vertex).getInfo();

				for (String value : target.getValues())
				{
					Rule rule = new Rule(dataset.getMetadata());
					Attribute attribute = metadata.get(term.getAttribute());

					if (attribute instanceof ContinuousAttribute)
					{
						ThresholdHelper helper = new ThresholdHelper(metadata);
						Vertex<Term> v =
							helper.create((ContinuousAttribute) attribute,
								(Vertex<Term>) vertex, dataset.getInstances(),
								value);

						rule.add(v);
					}
					else
					{
						rule.add(vertex);
					}

					// the heuristic value of the term for a class value
					// corresponds to the quality of a rule
					rule.setConsequent(metadata.getLabel().toLabel(value));
					Cost cost = function.evaluate(rule);

					vertex.setHeuristicValue(
						new HeuristicValue(cost.getValue()),
						target.getIndex(value));
				}
			}
			else if (!vertex.getLabel().equals(VIRTUAL_VERTEX))
			{
				double count = dataset.getInstances(vertex.getLabel()).size();

				vertex.setHeuristicValue(new HeuristicValue(count
					/ dataset.getSize()));
			}
		}
		*/
	}
}