/*
 * $Id: MaxMinStagnationTest.java 2339 2011-01-25 11:35:30Z febo2 $ 
 */

package net.sf.myra.cantminer.sequential;

import java.util.Collection;
import java.util.logging.Logger;

import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Environment;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.PheromoneUpdater;
import net.sf.myra.framework.StopCondition;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2339 $ $Date:: 2011-01-25 11:35:30#$
 */
public class MaxMinStagnationTest
	implements StopCondition
{
	/**
	 * The logger instance.
	 */
	private Logger logger = Logger.getLogger(MaxMinStagnationTest.class
		.getName());

	@SuppressWarnings("unchecked")
	public boolean isReached(Environment environment)
	{
		PheromoneUpdater updater =
			environment.getScheduler().getPheromoneUpdater();

		if (updater instanceof MaxMinPheromoneUpdater)
		{
			double tMin = ((MaxMinPheromoneUpdater) updater).getTMin();
			double tMax = ((MaxMinPheromoneUpdater) updater).getTMax();

			Graph<Term> graph =
				(Graph<Term>) environment.getColony().getTrailFactory()
					.getGraph();
			Collection<Vertex<Term>> vertices = graph.getVertices();
			Rule rule = (Rule) environment.getColony().getCurrent().getTrail();

			if (rule != null)
			{
				int levels = rule.getSize();

				for (int i = 0; i < levels; i++)
				{
					if (!hasConverged(vertices, i, tMin, tMax))
					{
						return false;
					}
				}

				logger.info("Search has converged at iteration "
					+ environment.getIteration());

				return true;
			}
			else
			{
				return false;
			}
		}

		throw new IllegalArgumentException(
			"Not a max-min pheromone updater instance: "
				+ updater.getClass().getName());
	}

	private boolean hasConverged(Collection<Vertex<Term>> vertices, int level,
		double tMin, double tMax)
	{
		int minCount = 0;
		int maxCount = 0;

		for (Vertex<Term> vertex : vertices)
		{
			double value = precision(vertex.getPheromone(level).getValue());

			if (value == precision(tMin))
			{
				minCount++;
			}

			if (value == precision(tMax))
			{
				maxCount++;
			}
		}

		return (minCount == (vertices.size() - 1)) && (maxCount == 1);
	}

	/**
	 * Returns the double value with up to four decimal digits.
	 * 
	 * @param value the value to be converted.
	 * 
	 * @return the double value with up to four decimal digits.
	 */
	private final double precision(double value)
	{
		int v = (int) (value * 100);
		return v / 100.0;
	}
}