/*
 * $Id: SequentialPheromoneUpdater.java 2339 2011-01-25 11:35:30Z febo2 $ 
 */

package net.sf.myra.cantminer.sequential;

import java.util.ArrayList;

import net.sf.myra.antminer.DefaultPheromoneUpdater;
import net.sf.myra.framework.Colony;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.Pheromone;
import net.sf.myra.framework.Trail;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision: 2339 $ $Date:: 2011-01-25 11:35:30#$
 */
public class SequentialPheromoneUpdater
	extends DefaultPheromoneUpdater
{
	@Override
	public void evaporate(Colony colony)
	{
		Graph<?> graph = colony.getTrailFactory().getGraph();
		ArrayList<Double> total = new ArrayList<Double>();

		for (Vertex<?> vertex : graph.getVertices())
		{
			for (int i = 0; i < vertex.levels(); i++)
			{
				double value = vertex.getPheromone(i).getValue();

				if (i < total.size())
				{
					total.set(i, total.get(i) + value);
				}
				else
				{
					total.add(value);
				}
			}
		}

		for (Vertex<?> vertex : graph.getVertices())
		{
			for (int i = 0; i < vertex.levels(); i++)
			{
				double value = vertex.getPheromone(i).getValue();
				vertex.getPheromone(i).setValue(value / total.get(i));
			}
		}
	}

	@Override
	public void update(Trail trail, Colony colony)
	{
		Graph<?> graph = colony.getTrailFactory().getGraph();
		int levels = trail.getSize();

		// ensures that all vertices have the same number of levels

		for (Vertex<?> vertex : graph.getVertices())
		{
			for (int i = 0; i < levels; i++)
			{
				vertex.setPheromone(vertex.getPheromone(i), i);
			}
		}

		// updates the pheromone value of the used terms (vertices)

		double delta = trail.getCost().getValue();
		int index = 0;

		for (Vertex<?> vertex : trail.getVertices())
		{
			Vertex<?> v = graph.get(vertex.getLabel());
			Pheromone pheromone = v.getPheromone(index);
			double value = pheromone.getValue();
			pheromone.setValue(value + (value * delta));
			index++;
		}
	}
}