/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import net.sf.myra.framework.LocalSearch;
import net.sf.myra.framework.ObjectiveFunction;

/**
 * This class provides a skeletal implementation of a rule prune procedure
 * based on the <code>LocalSearch</code> interface. This implementation uses
 * a <code>ObjectiveFunction</code> instance to evaluate the candidate solutions
 * generate during the pruning procedure. The <code>ClassAssignator</code>
 * instance is used to determine the consequent of the rule.
 * 
 * @see LocalSearch
 * @see ObjectiveFunction
 * @see ClassAssignator
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public abstract class AbstractPruner
	implements LocalSearch
{
	/**
	 * Reference to the <code>ObjectiveFunction</code> instance used to
	 * evaluate candidate rules.
	 */
	private AbstractObjectiveFunction function;

	/**
	 * Reference to the <code>ClassAssignator</code> instance used to
	 * compute the rule consequent.
	 */
	private ClassAssignator assignator;

	/**
	 * Default constructor.
	 * 
	 * @param function the <code>ObjectiveFunction</code> instance.
	 * @param assignator the <code>ClassAssignator</code> instance.
	 */
	public AbstractPruner(AbstractObjectiveFunction function,
			ClassAssignator assignator)
	{
		this.function = function;
		this.assignator = assignator;
	}

	/**
	 * Returns the <code>ObjectiveFunction</code> instance.
	 * 
	 * @return the <code>ObjectiveFunction</code> instance.
	 */
	public AbstractObjectiveFunction getFunction()
	{
		return function;
	}

	/**
	 * Returns the <code>ClassAssignator</code> instance.
	 * 
	 * @return the <code>ClassAssignator</code> instance.
	 */
	public ClassAssignator getClassAssignator()
	{
		return assignator;
	}
}