/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining;

import net.sf.myra.framework.Cost;
import net.sf.myra.framework.Trail;

/**
 * This class represents a prune procedure which removes the last term of the
 * rule until the rule quality improves.
 * 
 * @author Fernando Esteban Barril Otero
 */
public class BacktrackPruner
	extends AbstractPruner
{
	public BacktrackPruner(AbstractObjectiveFunction function,
			ClassAssignator assignator)
	{
		super(function, assignator);
	}

	public Trail explore(Trail trail)
	{
		Rule rule = (Rule) trail;
		ClassAssignator assignator = getClassAssignator();
		AbstractObjectiveFunction function = getFunction();

		assignator.assign(rule, function.getDataset());
		Cost quality = function.evaluate(rule);

		while (rule.getSize() > 1)
		{
			Rule improved = rule.clone();
			improved.remove(rule.getLast());

			assignator.assign(improved, function.getDataset());
			Cost c = function.evaluate(improved);

			// if the rule quality has improved or has the same quality
			if (c.compareTo(quality) > -1)
			{
				rule = improved;
				quality = c;
			}
			// otherwise we stop
			else
			{
				break;
			}
		}

		return rule;
	}
}