/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import java.util.List;
import net.sf.myra.datamining.ClassAssignator;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ClassFrequencyAssignator
	implements ClassAssignator
{
	public void assign(Rule rule, Dataset dataset)
	{
		if (!(rule instanceof ProbabilisticRule))
		{
			throw new IllegalArgumentException("ProbabilisticRule expected: "
				+ rule.getClass());
		}

		assign((ProbabilisticRule) rule, dataset);
	}

	/**
	 * Assigns the class probability array to the specified rule.
	 * 
	 * @param rule the rule to assign the class probability array.
	 * @param dataset the dataset used in the evaluation.
	 */
	public void assign(ProbabilisticRule rule, Dataset dataset)
	{
		List<Instance> covered = Evaluator.findCoveredCases(rule, dataset);
		double[] probabilities =
			new double[dataset.getMetadata().getLabel().length()];

		for (Instance instance : covered)
		{
			boolean[] flags = instance.labels();

			for (int i = 0; i < flags.length; i++)
			{
				if (flags[i])
				{
					probabilities[i]++;
				}
			}
		}

		for (int i = 0; i < probabilities.length; i++)
		{
			probabilities[i] /= covered.size();
		}

		rule.setProbabiblities(probabilities);
	}
}