/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Ant;
import net.sf.myra.framework.Colony;
import net.sf.myra.framework.DaemonAction;

/**
 * This class represents a daemon action responsible to evaluate the
 * confidence factor of the rules discovered by ants in the current
 * iteration.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ConfidenceEvaluator
	implements DaemonAction
{
	/**
	 * The current dataset.
	 */
	private Dataset dataset;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the current dataset reference.
	 */
	public ConfidenceEvaluator(Dataset dataset)
	{
		this.dataset = dataset;
	}

	/**
	 * Evaluated the confidence factor of rules discovered by ants in the
	 * specified colony.
	 * 
	 * @param colony the colony of ants.
	 */
	public void perform(Colony colony)
	{
		for (Ant ant : colony.getPopulation())
		{
			Rule rule = (Rule) ant.getTrail();
			rule.setConfidence(Evaluator.getConfidenceFactor(rule, dataset));
		}
	}
}