/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-20012 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining;

import java.util.List;

import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;

/**
 * @author Fernando Esteban Barril Otero
 */
public class CoveredClassAssignator
	implements ClassAssignator
{
	/**
	 * Assigns all class labels of the covered examples to the rule consequent.
	 * 
	 * @param rule the rule to assign the consequent.
	 * @param dataset the dataset used in the evaluation.
	 */
	public void assign(Rule rule, Dataset dataset)
	{
		List<Instance> covered = Evaluator.findCoveredCases(rule, dataset);

		if (!covered.isEmpty())
		{
			rule.setConsequent(Evaluator.getPresentLabel(covered));
		}
		else
		{
			rule.setConsequent(dataset.getMetadata().getLabel()
				.toLabel(new String[0]));
		}
	}
}