/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import java.util.logging.Logger;

import net.sf.myra.framework.Configuration;
import net.sf.myra.framework.Environment;
import net.sf.myra.framework.StopCondition;

/**
 * This class represents the iteration stop condition. It tests if the
 * colony of ants have reach a specified number of iterations.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date: 2008-02-08 15:54:20#$
 */
public class IterationTest
	implements StopCondition
{
	/**
	 * The property key under which the maximum iteration value is stored.
	 */
	public static final String MAXIMUM_ITERATIONS =
		"net.sf.myra.datamining.iteration";

	/**
	 * The default number of iterations.
	 */
	public static final int DEFAULT_MAXIMUM_ITERATIONS = 1500;
	
	/**
	 * The logger instance.
	 */
	private Logger logger = Logger.getLogger(IterationTest.class.getName());

	/**
	 * The maximum number of iterations. 
	 */
	private int maximum;

	/**
	 * Creates a new <code>IterationTest</code> instance.
	 */
	public IterationTest()
	{
		this(Configuration.getIntProperty(MAXIMUM_ITERATIONS,
			DEFAULT_MAXIMUM_ITERATIONS));
	}

	/**
	 * Creates a new <code>IterationTest</code> instance.
	 * 
	 * @param maximum the maximum number of iterations.
	 */
	public IterationTest(int maximum)
	{
		this.maximum = maximum;
	}

	public boolean isReached(Environment environment)
	{
		boolean reached = (environment.getIteration() >= maximum);
		
		if (reached)
		{
			logger.info("The maximum number of iteration was reached.");
		}
		
		return reached;
	}
}