/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import java.util.Collections;
import java.util.List;

import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;

/**
 * This class represents a <code>ClassAssignator</code> that determines a rule
 * predicted class by the majority of examples covered by the rule.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class MajorityClassAssignator
	implements ClassAssignator
{
	/**
	 * Assigns the rule's class value based on the majority class of the
	 * covered examples by the rule.
	 */
	@SuppressWarnings("unchecked")
	public void assign(Rule rule, Dataset dataset)
	{
		Metadata metadata = dataset.getMetadata();
		List<Instance> covered =
			Evaluator.findCoveredCases(rule, dataset.getInstances());
		Label majority = null;

		if (covered.isEmpty())
		{
			majority = metadata.getLabel().toLabel(Collections.EMPTY_LIST);
		}
		else
		{
			majority =
				Evaluator.findMajorityClassValue(covered, metadata
					.getIndex(metadata.getTarget().getName()));
		}

		rule.setConsequent(majority);
	}
}