/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Arrays;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ProbabilisticRule
	extends Rule
{
	private static final long serialVersionUID = 2956641747941828606L;

	/**
	 * The default number format instance (to format the 'toString' output).
	 */
	private static final NumberFormat FORMATTER = new DecimalFormat("0.00");

	/**
	 * The class frequency probabilities of this rule.
	 */
	private double[] probabiblities;

	/**
	 * Default constructor.
	 * 
	 * @param metadata
	 */
	public ProbabilisticRule(Metadata metadata)
	{
		this(metadata.getPredictor().size(), metadata.getLabel());
	}

	/**
	 * Creates a new <code>Rule</code> instance.
	 * 
	 * @param size the maximum size of the rule antecedent.
	 * @param prototype the label prototype instance.
	 */
	public ProbabilisticRule(int size, Label prototype)
	{
		super(size, prototype);
		probabiblities = new double[0];
	}

	@Override
	public final Label getConsequent()
	{
		throw new UnsupportedOperationException();
	}

	@Override
	public final void setConsequent(Label consequent)
	{
		throw new UnsupportedOperationException();
	}

	/**
	 * Returns the class probability array.
	 * 
	 * @return the class probability array.
	 */
	public double[] getProbabiblities()
	{
		return probabiblities;
	}

	/**
	 * Sets the class probability array.
	 * 
	 * @param probabiblities the class probabiblity array to set.
	 */
	public void setProbabiblities(double[] probabiblities)
	{
		this.probabiblities = probabiblities;
	}

	@Override
	public String toString()
	{
		StringBuffer buffer = new StringBuffer(super.toString());

		if (probabiblities.length != 0)
		{
			buffer.delete(buffer.indexOf("<undefined>"), buffer.length());
			buffer.append("[");
			Label prototytpe = super.getConsequent();

			for (int i = 0; i < probabiblities.length; i++)
			{
				if (probabiblities[i] > 0)
				{
					buffer.append(prototytpe.names()[i]);
					buffer.append(":");
					buffer.append(FORMATTER.format(probabiblities[i]));
					buffer.append(",");
				}
			}

			buffer.deleteCharAt(buffer.length() - 1);
			buffer.append("]");
		}

		return buffer.toString();
	}

	@Override
	public ProbabilisticRule clone()
	{
		ProbabilisticRule clone = (ProbabilisticRule) super.clone();
		clone.probabiblities = probabiblities.clone();
		return clone;
	}

	@Override
	public boolean equals(Object o)
	{
		return equals((ProbabilisticRule) o);
	}

	/**
	 * Compares this rule against the specified rule for equality.
	 * 
	 * @param o the rule to be compared to.
	 * 
	 * @return <code>true</code> if both rules are equal; <code>false</code>
	 *         otherwise.
	 */
	private boolean equals(ProbabilisticRule o)
	{
		return super.equals((Rule) o)
			&& Arrays.equals(probabiblities, o.probabiblities);
	}
}