/*
 * $Id$ 
 */

package net.sf.myra.datamining;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ProbabilisticSequentialRule
	extends ProbabilisticRule
{
	private static final long serialVersionUID = -3527089196536710086L;

	/**
	 * The list of vertices.
	 */
	private LinkedList<Vertex<?>> vertices;

	/**
	 * The last vertex added to the trail.
	 */
	private Vertex<?> last;

	/**
	 * Default constructor.
	 * 
	 * @param metadata
	 */
	public ProbabilisticSequentialRule(Metadata metadata)
	{
		this(metadata.getPredictor().size(), metadata.getLabel());
	}

	/**
	 * Creates a new <code>Rule</code> instance.
	 * 
	 * @param size the maximum size of the rule antecedent.
	 * @param prototype the label prototype instance.
	 */
	public ProbabilisticSequentialRule(int size, Label prototype)
	{
		super(size, prototype);
		vertices = new LinkedList<Vertex<?>>();
	}

	@Override
	public void add(Vertex<?> vertex)
	{
		vertices.add(vertex);
		last = vertex;
	}

	@Override
	public Vertex<?> getLast()
	{
		return last;
	}

	@Override
	public Collection<Vertex<?>> getVertices()
	{
		return Collections.unmodifiableList(vertices);
	}

	@Override
	public boolean isEmpty()
	{
		return vertices.isEmpty();
	}

	@Override
	public void remove(Vertex<?> vertex)
	{
		vertices.remove(vertex);
		last = vertices.getLast();
		// if the trail is changed, the cost is no longer valid
		setCost(null);
	}

	@Override
	public ProbabilisticSequentialRule clone()
	{
		ProbabilisticSequentialRule trail =
			(ProbabilisticSequentialRule) super.clone();
		trail.vertices = new LinkedList<Vertex<?>>(vertices);
		return trail;
	}

	@Override
	public boolean equals(Object o)
	{
		return equals((ProbabilisticSequentialRule) o);
	}

	private boolean equals(ProbabilisticSequentialRule o)
	{
		if (this == o)
		{
			return true;
		}
		else if (o == null)
		{
			return false;
		}

		return vertices.equals(o.getVertices())
			&& Arrays.equals(getProbabiblities(), o.getProbabiblities());
	}

	@SuppressWarnings("unchecked")
	@Override
	public Term[] getAntecedent()
	{
		Term[] antecedent = new Term[vertices.size()];
		int index = 0;

		for (Vertex<?> vertex : vertices)
		{
			antecedent[index] = ((Vertex<Term>) vertex).getInfo();
			index++;
		}

		return antecedent;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean isCovered(Instance instance)
	{
		double[] values = instance.values();

		for (Vertex<?> vertex : vertices)
		{
			Term term = ((Vertex<Term>) vertex).getInfo();

			if (!term.satisfies(values[term.getIndex()]))
			{
				return false;
			}
		}

		return true;
	}
}