/*
 * $Id$ 
 */

package net.sf.myra.datamining;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class SequentialRule
	extends Rule
{
	private static final long serialVersionUID = 5142882947050705557L;

	/**
	 * The list of vertices.
	 */
	private LinkedList<Vertex<?>> vertices;

	/**
	 * The last vertex added to the trail.
	 */
	private Vertex<?> last;

	/**
	 * Creates a new <code>DefaultTrail</code> instance.
	 */
	public SequentialRule(Metadata metadata)
	{
		super(metadata);
		vertices = new LinkedList<Vertex<?>>();
	}

	@Override
	public void add(Vertex<?> vertex)
	{
		vertices.add(vertex);
		last = vertex;
	}

	@Override
	public Vertex<?> getLast()
	{
		return last;
	}

	@Override
	public Collection<Vertex<?>> getVertices()
	{
		return Collections.unmodifiableList(vertices);
	}

	@Override
	public boolean isEmpty()
	{
		return vertices.isEmpty();
	}

	@Override
	public void remove(Vertex<?> vertex)
	{
		vertices.remove(vertex);
		last = vertices.getLast();
		// if the trail is changed, the cost is no longer valid
		setCost(null);
	}

	@Override
	public SequentialRule clone()
	{
		SequentialRule trail = (SequentialRule) super.clone();
		trail.vertices = new LinkedList<Vertex<?>>(vertices);
		return trail;
	}

	@Override
	public boolean equals(Object o)
	{
		return equals((SequentialRule) o);
	}

	private boolean equals(SequentialRule o)
	{
		if (this == o)
		{
			return true;
		}
		else if (o == null)
		{
			return false;
		}

		Label c1 = getConsequent();
		Label c2 = o.getConsequent();

		// sanity check: makes sure the consequent is not null
		if ((c1 == null) && (c2 != null))
		{
			return false;
		}

		return ((c1 == c2) || c1.equals(c2))
			&& vertices.equals(o.getVertices());
	}

	@SuppressWarnings("unchecked")
	@Override
	public Term[] getAntecedent()
	{
		Term[] antecedent = new Term[vertices.size()];
		int index = 0;

		for (Vertex<?> vertex : vertices)
		{
			antecedent[index] = ((Vertex<Term>) vertex).getInfo();
			index++;
		}

		return antecedent;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean isCovered(Instance instance)
	{
		double[] values = instance.values();

		for (Vertex<?> vertex : vertices)
		{
			Term term = ((Vertex<Term>) vertex).getInfo();

			if (!term.satisfies(values[term.getIndex()]))
			{
				return false;
			}
		}

		return true;
	}
}