/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining;

/**
 * This class represents a unit of quality measurement for <code>Model</code>
 * instances.
 * 
 * @see Measure
 *  
 * @author Fernando Esteban Barril Otero
 */
public class Unit<V extends Comparable<V>>
	implements Comparable<Unit<V>>
{
	/**
	 * The quality measurement value.
	 */
	private V value;

	/**
	 * Default constructor.
	 * 
	 * @param value the quality measurement value.
	 */
	public Unit(V value)
	{
		this.value = value;
	}

	/**
	 * Return the quality measurement value.
	 * 
	 * @return the quality measurement value.
	 */
	public V getValue()
	{
		return value;
	}

	public String toString()
	{
		return value.toString();
	}

	public int compareTo(Unit<V> o)
	{
		return value.compareTo(o.value);
	}

	public double doubleValue()
	{
		if (Number.class.isAssignableFrom(getValue().getClass()))
		{
			return ((Number) getValue()).doubleValue();
		}
		else if (DoubleValued.class.isAssignableFrom(getValue().getClass()))
		{
			return ((DoubleValued) getValue()).doubleValue();
		}

		throw new UnsupportedOperationException("Not a number: "
			+ getValue().getClass());
	}

	/**
	 * Interface to indicate a unit value which can be represented as a double.
	 */
	public interface DoubleValued
	{
		/**
		 * Returns the double representation of the value.
		 * 
		 * @return the double representation of the value.
		 */
		public double doubleValue();
	}
}