
package net.sf.myra.datamining;

import java.util.HashMap;
import java.util.Map;

import net.sf.myra.datamining.data.ClassHierarchy;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Node;
import net.sf.myra.framework.Configuration;

public class UsefulnessHelper
{
	public static final String USEFULNESS = "net.sf.myra.datamining.usefulness";

	public static final String FACTOR =
		"net.sf.myra.datamining.usefulness.factor";

	private double factor = Configuration.getDoubleProperty(FACTOR, 1.0);

	private Map<String, Double> usefulness;

	public UsefulnessHelper(Dataset dataset)
	{
		ClassHierarchy hierarchy = dataset.getMetadata().getClassHierarchy();
		usefulness = new HashMap<String, Double>();
		double max = 0.0;

		for (Node node : hierarchy.flat())
		{
			double value = size(node, dataset);
			usefulness.put(node.getLabel(), value);

			if (value > max)
			{
				max = value;
			}
		}

		for (Node node : hierarchy.flat())
		{
			double value = usefulness.get(node.getLabel());
			usefulness.put(node.getLabel(), 1 - (value / max));
		}
	}

	private double size(Node node, Dataset dataset)
	{
		if (dataset.getInstances(node.getLabel()).size() > 0)
		{
			return factor
				* (Math.log(1 + node.getDescendants().size()) / Math.log(2.0));
		}
		// the class label does not occur in the training set
		return 0;
	}

	public double get(String label)
	{
		return usefulness.get(label);
	}
}