/*
 * $Id$ 
 */

package net.sf.myra.datamining.data;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                    #$
 */
public class DistanceIntervalBuilder
	extends AbstractDistanceBasedBuilder
{
	/**
	 * Default constructor.
	 * 
	 * @param metadata the dataset metadata.
	 */
	public DistanceIntervalBuilder(Metadata metadata)
	{
		super(metadata);
	}

	@Override
	protected CutPoint[] create(Candidate[] candidates, boolean filter)
	{
		CutPoint point = new CutPoint();
		point.gain = Double.NEGATIVE_INFINITY;
		point.impurity_s = variance(candidates, 0, candidates.length);

		if (candidates.length > 1 && candidates.length >= (minimumLimit * 2))
		{
			boolean force = false;

			// selects the best cut point

			for (int i = 0; i < (candidates.length - 1); i++)
			{
				if ((candidates[i].value != candidates[i + 1].value) || force)
				{
					int t = i + 1;

					if (t >= minimumLimit)
					{
						force = false;

						// if the previous value is equal to the threshold value
						// we skip forward
						while (t < (candidates.length - 1)
							&& candidates[t - 1].value == candidates[t].value)
						{
							t++;
						}

						// if the upper partition is smaller than the limit, we
						// move backwards
						while ((candidates.length - 1 - t) < minimumLimit
							|| (t > 0 && candidates[t].value == candidates[t - 1].value))
						{
							t--;
						}

						if (t < minimumLimit)
						{
							force = true;
						}
						else
						{
							double v1 = variance(candidates, 0, t);
							double v2 =
								variance(candidates, t, candidates.length);

							double gain =
								point.impurity_s
									- ((t / (double) candidates.length) * v1)
									- (((candidates.length - t) / (double) candidates.length) * v2);

							if (gain > point.gain)
							{
								point.gain = gain;
								point.threshold = t;
								point.impurity_s1 = v1;
								point.impurity_s2 = v2;
								point.value = candidates[t].value;
							}
						}
					}
					else
					{
						force = true;
					}
				}
			}
		}

		return new CutPoint[] { point };
	}
}