/*
 * $Id$ 
 */

package net.sf.myra.datamining.data;

import net.sf.myra.framework.Graph;
import net.sf.myra.framework.HeuristicInformation;
import net.sf.myra.framework.HeuristicValue;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class EntropyHeuristicInformation
	implements HeuristicInformation
{
	/**
	 * The current data set.
	 */
	private Dataset dataset;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the current data set.
	 */
	public EntropyHeuristicInformation(Dataset dataset)
	{
		this.dataset = dataset;
	}

	public void initialize(Graph<?> graph)
	{
		initialize(graph, new EntropyContent(dataset));
	}

	/**
	 * Initializes the vertices' heuristic information of the specified graph.
	 * The entropy information is obtained form the specified
	 * <code>EntropyContent</code> helper, so there is no overhead in computing
	 * the information again.
	 * 
	 * @param graph the construction graph.
	 * @param content the <code>EntropyContent</code> helper.
	 */
	public void initialize(Graph<?> graph, EntropyContent content)
	{
		double log_k =
			Math.log(((NominalAttribute) dataset.getMetadata().getTarget())
				.getValues().size()) / Math.log(2.0);

		for (Vertex<?> vertex : graph.getVertices())
		{
			if (content.contains(vertex.getLabel()))
			{
				double entropy =
					(content.frequency(vertex.getLabel()) > 0) ? content
						.entropy(vertex.getLabel()) : log_k;

				if (Double.isInfinite(entropy))
				{
					vertex.setHeuristicValue(new HeuristicValue(0));
				}
				else
				{
					vertex
						.setHeuristicValue(new HeuristicValue(log_k - entropy));
				}
			}
		}
	}
}