/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.function;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

public class AccuracyFunction
	extends AbstractObjectiveFunction
{
	/**
	 * Default constructor.
	 * 
	 * @param dataset the dataset used in the evaluation.
	 */
	public AccuracyFunction(Dataset dataset)
	{
		super(dataset);
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		Dataset dataset = getDataset();

		if (rule.getConsequent().length() > 0)
		{
			ConfusionMatrix m = Evaluator.getConfusionMatrix(rule, dataset);
			return new NumericCost(m.getAccuracy());
		}

		return new NumericCost(0.0);
	}
}