/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.function;

import static net.sf.myra.datamining.ConfusionMatrix.FP;
import static net.sf.myra.datamining.ConfusionMatrix.TP;
import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.ErrorBasedMetric;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * This class represents a objective function based on the pressimistic error
 * rate used by C4.5 algorithm.
 * 
 * @author Fernando Esteban Barril Otero
 */
public class ErrorBasedFunction
	extends AbstractObjectiveFunction
{
	/**
	 * Default constructor.
	 */
	public ErrorBasedFunction(Dataset dataset)
	{
		super(dataset);
	}

	public Cost evaluate(Trail trail)
	{
		ConfusionMatrix matrix =
			Evaluator.getConfusionMatrix((Rule) trail, getDataset());
		double error =
			ErrorBasedMetric.estimate(matrix.get(FP),
				matrix.get(TP) + matrix.get(FP));

		return new NumericCost(1.0 - error);
	}
}