/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero, Matthew Medland
 */

package net.sf.myra.datamining.function;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Configuration;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * @author Matthew Medland
 */
public class RelativeCostMeasureFunction
	extends AbstractObjectiveFunction
{
	public static final String Cr =
		"net.sf.myra.datamining.relativecostmeasure.cr";

	private double cr;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the dataset used in the evaluation.
	 */
	public RelativeCostMeasureFunction(Dataset dataset)
	{
		this(dataset, Configuration.getDoubleProperty(Cr, 0.5));
	}

	/**
	 * Construction with parameter.
	 * 
	 * @param dataset the dataset used in the evaluation.
	 * @param cr the cr value to be used
	 */
	public RelativeCostMeasureFunction(Dataset dataset, double cr)
	{
		super(dataset);
		this.cr = cr;
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		Dataset dataset = getDataset();

		double relativeCostMeasure = 0.0;

		if (rule.getConsequent().length() > 0)
		{
			ConfusionMatrix m = Evaluator.getConfusionMatrix(rule, dataset);

			double true_pos_rate = m.getRecall();
			double false_positive_rate =
				(double) m.get(ConfusionMatrix.FP)
					/ ((double) m.get(ConfusionMatrix.FP) + m
						.get(ConfusionMatrix.TN));

			relativeCostMeasure =
				(cr * true_pos_rate) - ((1 - cr) * false_positive_rate);
		}

		return new NumericCost(Double.isNaN(relativeCostMeasure) ? 0.0
			: relativeCostMeasure);
	}
}
