/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.function;

import java.util.logging.Logger;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * This class represents the sensitivity <code>x</code> specificity rule
 * quality measure.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date: 2008-02-17 15:33:56#$
 */
public class SensitivitySpecificityFunction
	extends AbstractObjectiveFunction
{
	private static final double CORRECTION = 0.01;
	/**
	 * The logger instance.
	 */
	private Logger logger =
		Logger.getLogger(SensitivitySpecificityFunction.class.getName());

	/**
	 * Creates a new <code>SensitivitySpecificityFunction</code> instance.
	 * 
	 * @param dataset the dataset reference.
	 */
	public SensitivitySpecificityFunction(Dataset dataset)
	{
		super(dataset);
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		Dataset dataset = getDataset();

		double cost = 0.0;

		if (rule.getConsequent().length() > 0)
		{
			ConfusionMatrix m = Evaluator.getConfusionMatrix(rule, dataset);
			cost = m.getSensitivity() * m.getSpecificity();

			if (cost == 0 && !rule.isEmpty())
			{
				logger.info("Rule has a zero cost (sen=" + m.getSensitivity()
					+ ",spe=" + m.getSpecificity() + "): " + rule);
				
				if (m.getSensitivity() == 0)
				{
					cost = CORRECTION * m.getSpecificity();
				}
				else if (m.getSpecificity() == 0)
				{
					cost = m.getSensitivity() * CORRECTION;
				} 
			}
		}

		// checks if the cost value is a Not-a-Number (NaN)
		return new NumericCost(Double.isNaN(cost) ? 0.0 : cost);
	}
}