/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero and Khalid Salama
 */

package net.sf.myra.datamining.function.hierarchical;

import static net.sf.myra.datamining.ConfusionMatrix.FN;
import static net.sf.myra.datamining.ConfusionMatrix.FP;
import static net.sf.myra.datamining.ConfusionMatrix.TN;
import static net.sf.myra.datamining.ConfusionMatrix.TP;

import java.util.List;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;

/**	
 * @author Fernando Esteban Barril Otero
 */
public abstract class LocalFunction
	extends AbstractObjectiveFunction
{
	public LocalFunction(Dataset dataset)
	{
		super(dataset);
	}

	protected ConfusionMatrix getConfusionMatrix(Rule rule, String label,
		List<Instance> instances)
	{
		Label consequent = rule.getConsequent();
		boolean active = consequent.active(label);
		ConfusionMatrix matrix = new ConfusionMatrix();

		for (Instance instance : instances)
		{
			boolean isCovered = rule.isCovered(instance);

			if (active)
			{
				if (instance.belongs(label))
				{
					matrix.increment(isCovered ? TP : FN);
				}
				else
				{
					matrix.increment(isCovered ? FP : TN);
				}
			}
			else
			{
				if (instance.belongs(label))
				{
					matrix.increment(isCovered ? FP : TN);
				}
				else
				{
					matrix.increment(isCovered ? TP : FN);
				}
			}
		}

		return matrix;
	}
}