/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.function.hierarchical;

import java.util.List;
import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.ProbabilisticRule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.DistanceContent;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class VarianceFunction
	extends AbstractObjectiveFunction
{
	/**
	 * The maximum (worst) variance value.
	 */
	private final double maximum;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the dataset used in the evaluation.
	 */
	public VarianceFunction(Dataset dataset)
	{
		super(dataset);

		maximum =
			Math.log(dataset.getMetadata().getTarget().getValues().size())
				/ Math.log(2.0);
	}

	public Cost evaluate(Trail trail)
	{
		ProbabilisticRule rule = (ProbabilisticRule) trail;
		List<Instance> covered = Evaluator.findCoveredCases(rule, getDataset());
		double[] mean = rule.getProbabiblities();

		return new NumericCost((maximum - DistanceContent.variance(covered,
			mean))
			/ maximum);
	}
}