/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.function.hierarchical;

import java.util.List;
import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.DistanceContent;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class VarianceGainFunction
	extends AbstractObjectiveFunction
{
	/**
	 * The variance of the complete dataset.
	 */
	private double variance;

	public VarianceGainFunction(Dataset dataset)
	{
		super(dataset);
		this.variance = DistanceContent.variance(dataset.getInstances());
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		Dataset dataset = getDataset();

		List<Instance> positive = Evaluator.findCoveredCases(rule, dataset);
		List<Instance> negative = Evaluator.findNotCoveredCases(rule, dataset);

		if (positive.size() == 0 || negative.size() == 0)
		{
			return new NumericCost(0.0);
		}

		double v1 = DistanceContent.variance(positive);
		double v2 = DistanceContent.variance(negative);

		return new NumericCost(variance
			- ((positive.size() / (double) dataset.getSize()) * v1)
			- ((negative.size() / (double) dataset.getSize()) * v2));
	}
	
	/**
	 * Returns the variance of the dataset.
	 * 
	 * @return the variance of the dataset.
	 */
	public double getVariance()
	{
		return variance;
	}
}