/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.function.hierarchical;

import java.util.List;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.UsefulnessHelper;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.framework.Configuration;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**
 * This class represents the hierarchical F-measure function.
 * 
 * @author Fernando Esteban Barril Otero
 */
public class hFmeasure
	extends AbstractObjectiveFunction
{
	public static final String BETA = "net.sf.myra.datamining.hfmeasure.beta";

	private double beta;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the dataset used in the evaluation.
	 */
	public hFmeasure(Dataset dataset)
	{
		super(dataset);
		beta = Math.pow(Configuration.getDoubleProperty(BETA, 1.0), 2);
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		List<Instance> instances = getDataset().getInstances();
		UsefulnessHelper helper = null;

		if (Configuration.getBooleanProperty(UsefulnessHelper.USEFULNESS))
		{
			helper = new UsefulnessHelper(getDataset());
		}

		ConfusionMatrix matrix =
			(helper == null) ? Evaluator.getConfusionMatrix(rule, instances)
				: Evaluator.getConfusionMatrix(rule, instances, helper);

		double hF =
			(1 + beta)
				* ((matrix.getPrecision() * matrix.getRecall()) / ((beta * matrix
					.getPrecision()) + matrix.getRecall()));

		return new NumericCost(NumericCost.isNumber(hF) ? hF : 0.0);
	}
}