/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero and Khalid Salama
 */

package net.sf.myra.datamining.function.hierarchical;

import static net.sf.myra.datamining.ConfusionMatrix.FN;
import static net.sf.myra.datamining.ConfusionMatrix.FP;
import static net.sf.myra.datamining.ConfusionMatrix.TP;

import java.util.List;

import net.sf.myra.datamining.AbstractObjectiveFunction;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.UsefulnessHelper;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.framework.Configuration;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**	
 * @author Fernando Esteban Barril Otero
 * @author Khalid Salama
 */
public class hJaccard
	extends AbstractObjectiveFunction
{
	public hJaccard(Dataset dataset)
	{
		super(dataset);
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		List<Instance> instances = getDataset().getInstances();
		UsefulnessHelper helper = null;

		if (Configuration.getBooleanProperty(UsefulnessHelper.USEFULNESS))
		{
			helper = new UsefulnessHelper(getDataset());
		}

		ConfusionMatrix m =
			(helper == null) ? Evaluator.getConfusionMatrix(rule, instances)
				: Evaluator.getConfusionMatrix(rule, instances, helper);

		double cost = m.get(TP) / (m.get(TP) + m.get(FP) + m.get(FN));

		return new NumericCost(NumericCost.isNumber(cost) ? cost : 0.0);
	}
}