/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero and Khalid Salama
 */

package net.sf.myra.datamining.function.hierarchical;

import static net.sf.myra.datamining.ConfusionMatrix.FN;
import static net.sf.myra.datamining.ConfusionMatrix.FP;
import static net.sf.myra.datamining.ConfusionMatrix.TN;
import static net.sf.myra.datamining.ConfusionMatrix.TP;

import java.util.List;

import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.framework.Trail;

/**	
 * @author Fernando Esteban Barril Otero
 * @author Khalid Salama
 */
public class hLocalJaccard
	extends LocalFunction
{
	public hLocalJaccard(Dataset dataset)
	{
		super(dataset);
	}

	public Cost evaluate(Trail trail)
	{
		Rule rule = (Rule) trail;
		Dataset dataset = getDataset();
		List<Instance> covered = Evaluator.findCoveredCases(rule, dataset);

		// the array of class labels present in the covered examples and
		// their frequency

		double[] probabilities =
			new double[dataset.getMetadata().getLabel().length()];

		for (Instance instance : covered)
		{
			boolean[] flags = instance.labels();

			for (int i = 0; i < flags.length; i++)
			{
				if (flags[i])
				{
					probabilities[i]++;
				}
			}
		}

		for (int i = 0; i < probabilities.length; i++)
		{
			probabilities[i] /= covered.size();
		}

		Label consequent = rule.getConsequent();
		String root =
			dataset.getMetadata().getClassHierarchy().getRoot().getLabel();
		ConfusionMatrix m = new ConfusionMatrix();

		double cost = 0.0;

		for (int i = 0; i < probabilities.length; i++)
		{
			String value = consequent.get(i);

			if (!value.equals(root) && probabilities[i] > 0)
			{
				m = getConfusionMatrix(rule, value, dataset.getInstances());

				double tp = m.get(TP);
				double tn = m.get(TN);
				double fp = m.get(FP);
				double fn = m.get(FN);

				double total = (tp + tn + fp + fn);
				double A = (tp + fp) / total;
				double B = (tp + fn) / total;
				double AB = tp / total;
				double nAB = tn / total;

				double jaccard = AB / (A + B - nAB);
				jaccard = (Double.isInfinite(jaccard) ? 0.0 : jaccard);

				if (consequent.active(i))
				{
					cost += (probabilities[i] * jaccard);
				}
				else
				{
					cost -= (probabilities[i] * jaccard);
				}
			}
		}

		return new NumericCost(NumericCost.isNumber(cost) ? cost : 0.0);
	}
}