/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.io;

import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.model.RuleSet;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                    #$
 */
public class FrequencyExporter
	extends ProbabilisticExporter
{
	@Override
	public void write(Dataset testSet, Model discovered)
	{
		if (!(discovered instanceof RuleSet))
		{
			throw new IllegalArgumentException("Rule set expected: "
				+ discovered.getClass());
		}

		RuleSet ruleSet = (RuleSet) discovered;
		Metadata metadata = testSet.getMetadata();
		Label prototype = metadata.getLabel();

		for (Instance instance : testSet)
		{
			double[] probability = new double[prototype.length()];
			int rules = 0;

			for (Rule rule : ruleSet.getRules())
			{
				if (rule.isCovered(instance))
				{
					Label predicted = rule.getConsequent();

					for (int i = 0; i < probability.length; i++)
					{
						if (predicted.active(i))
						{
							probability[i]++;
						}
					}

					rules++;
				}
			}

			for (int i = 0; i < probability.length; i++)
			{
				double p = probability[i];
				probability[i] = (p > 0 ? p / rules : p);
			}

			export(instance, probability);
		}
	}
}