/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.measure;

import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;

/**
 * This class represents the accuracy measure.
 * 
 * @see Model
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class AccuracyMeasure
	implements Measure
{
	public Unit<Double> evaluate(Dataset dataset, Model model)
	{
		int correct = 0;

		for (Instance instance : dataset)
		{
			if (instance.getLabel().getValues().size() > 1)
			{
				throw new UnsupportedOperationException(
					"Multi-label instace found.");
			}

			if (instance.getLabel().equals(model.classify(instance)))
			{
				correct++;
			}
		}

		return new Unit<Double>(correct == 0 ? 0.0
			: (correct / (double) dataset.getSize()));
	}
}