
package net.sf.myra.datamining.measure;

import static net.sf.myra.datamining.ConfusionMatrix.FN;
import static net.sf.myra.datamining.ConfusionMatrix.FP;
import static net.sf.myra.datamining.ConfusionMatrix.TN;
import static net.sf.myra.datamining.ConfusionMatrix.TP;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Configuration;

public class CorrectnessRateMeasure
	implements Measure
{
	public static final String RMAX =
		"net.sf.myra.datamining.measure.correctness.rmax";

	public static final String RMIN =
		"net.sf.myra.datamining.measure.correctness.rmin";

	public static final String W1 =
		"net.sf.myra.datamining.measure.correctness.w1";

	public static final String W2 =
		"net.sf.myra.datamining.measure.correctness.w2";

	public static final String W3 =
		"net.sf.myra.datamining.measure.correctness.w3";

	private final double R_MAX;

	private final double R_MIN;

	private final double w1;

	private final double w2;

	private final double w3;

	public CorrectnessRateMeasure()
	{
		R_MAX = Configuration.getDoubleProperty(RMAX, 50.0);
		R_MIN = Configuration.getDoubleProperty(RMIN, 60.0);
		w1 = Configuration.getDoubleProperty(W1, 0.6);
		w2 = Configuration.getDoubleProperty(W2, 0.1);
		w3 = Configuration.getDoubleProperty(W3, 0.3);
	}

	public Unit<?> evaluate(Dataset dataset, Model model)
	{
		ConfusionMatrix m = Evaluator.getConfusionMatrix(model, dataset);
		double correctness = -1;

		if ((m.get(FP) + m.get(TP) != 0) && (m.get(FN) + m.get(TP) != 0))
		{
			if (((m.get(TP) + m.get(FP) / dataset.getSize()) * 100 >= R_MIN)
				&& ((m.get(TP) + m.get(FP) / dataset.getSize()) * 100 >= R_MAX))
			{
				correctness =
					(w1 * rateOfCorrectness(m.get(TP), m.get(TN),
						m.get(TP) + m.get(FN), m.get(TN) + m.get(FP)))
						- (w2 * rateOfMissingChances(m.get(FN),
							m.get(TP) + m.get(FN)))
						- (w3 * rateOfFailure(m.get(FP), m.get(TP)));
			}
		}

		return new Unit<Double>(correctness);
	}

	public double rateOfCorrectness(double truePositive, double trueNegative,
		double actualPositive, double actualNegative)
	{
		return (truePositive + trueNegative)
			/ (actualPositive + actualNegative);
	}

	public double rateOfMissingChances(double falseNegative,
		double actualPositive)
	{
		return falseNegative / actualPositive;
	}

	public double rateOfFailure(double falsePositive, double truePositive)
	{
		return falsePositive / (falsePositive + truePositive);
	}
}