/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.measure;

import java.util.Collection;

import net.sf.myra.datamining.ErrorBasedMetric;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.model.AbstractRuleModel;

/**
 * @author Fernando Esteban Barril Otero
 */
public class ErrorBasedMeasure
	implements Measure
{
	public Unit<Double> evaluate(Dataset dataset, Model model)
	{
		// we only support rule list models

		if (!(model instanceof AbstractRuleModel))
		{
			throw new IllegalArgumentException(
				"Rule list model required, found: " + model.getClass());
		}

		AbstractRuleModel list = (AbstractRuleModel) model;
		Collection<Rule> rules = list.getRules();

		double[] coverage = new double[rules.size()];
		double[] errors = new double[rules.size()];

		for (Instance instance : dataset)
		{
			int index = 0;

			for (Rule rule : rules)
			{
				if (rule.isCovered(instance))
				{
					coverage[index]++;

					if (!instance.getLabel().equals(rule.getConsequent()))
					{
						errors[index]++;
					}
					
					break;
				}

				index++;
			}
		}

		double predicted = 0.0;

		for (int i = 0; i < coverage.length; i++)
		{
			predicted += ErrorBasedMetric.errors(errors[i], coverage[i]);
		}

		return new Unit<Double>(1.0 - (predicted / (double) dataset.getSize()));
	}
}