/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero and Matthew Medland
 */

package net.sf.myra.datamining.measure;

import static net.sf.myra.datamining.measure.MicroAveragedFMeasure.BETA;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Configuration;

/**
 * This class represents the macro averaged F-Measure metric.
 * 
 * @author Fernando Esteban Barril Otero
 */
public class MacroAveragedFMeasure
	implements Measure
{
	private double beta;

	public MacroAveragedFMeasure()
	{
		beta = Math.pow(Configuration.getDoubleProperty(BETA, 1.0), 2);
	}

	public Unit<Double> evaluate(Dataset dataset, Model model)
	{
		double weight =
			1.0 / ((double) dataset.getMetadata().getTarget().getValues()
				.size());
		double measure = 0.0;

		for (String label : dataset.getMetadata().getTarget().getValues())
		{
			ConfusionMatrix matrix =
				Evaluator.getConfusionMatrix(model, dataset, label);

			double precision = matrix.getPrecision();
			double recall = matrix.getRecall();

			double fmeasure =
				(beta + 1)
					* ((precision * recall) / ((beta * precision) + recall));

			measure += weight * (Double.isNaN(fmeasure) ? 0.0 : fmeasure);
		}

		return new Unit<Double>(measure);
	}
}