/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.measure.hierarchical;

import static net.sf.myra.datamining.ConfusionMatrix.FN;
import static net.sf.myra.datamining.ConfusionMatrix.TP;
import net.sf.myra.datamining.ConfusionMatrix;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.ClassHierarchy;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class LevelMeasure
	implements Measure
{
	public Unit<?> evaluate(Dataset dataset, Model model)
	{
		ClassHierarchy hierarchy = dataset.getMetadata().getClassHierarchy();
		ConfusionMatrix[] m = new ConfusionMatrix[hierarchy.getLevels() + 1];

		for (int i = 0; i < m.length; i++)
		{
			m[i] = new ConfusionMatrix();
		}

		for (Instance instance : dataset)
		{
			Label pLabel = model.classify(instance);
			Label tLabel = instance.getLabel();

			for (int i = 0; i < tLabel.length(); i++)
			{
				if (tLabel.active(i))
				{
					int level = hierarchy.getLevel(pLabel.get(i));
					m[level].increment(pLabel.active(i) ? TP : FN);
				}
			}
		}

		return new Unit<LevelRecall>(new LevelRecall(m));
	}

	private static class LevelRecall
		implements Comparable<LevelRecall>
	{
		private ConfusionMatrix[] levels;

		LevelRecall(ConfusionMatrix[] levels)
		{
			this.levels = levels;
		}

		@Override
		public String toString()
		{
			StringBuffer buffer = new StringBuffer();
			buffer.append("\n");

			for (int i = 0; i < levels.length; i++)
			{
				buffer.append("level ");
				buffer.append(i);
				buffer.append(": ");
				buffer.append(levels[i].getRecall());
				buffer.append("\n");
			}

			return buffer.toString();
		}

		public int compareTo(LevelRecall o)
		{
			throw new UnsupportedOperationException("Not implemented.");
		}
	}
}