/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.measure.hierarchical;

import java.util.List;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.function.hierarchical.VarianceGainFunction;
import net.sf.myra.datamining.model.AbstractRuleModel;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                    #$
 */
public class VarianceGainMeasure
	implements Measure
{
	public Unit<?> evaluate(Dataset dataset, Model model)
	{
		if (!(model instanceof AbstractRuleModel))
		{
			throw new IllegalArgumentException("Rule model expected: "
				+ model.getClass());
		}

		AbstractRuleModel rules = (AbstractRuleModel) model;
		Dataset local = dataset.clone();
		double gain = 0.0;

		for (Rule rule : rules.getRules())
		{
			List<Instance> covered = Evaluator.findCoveredCases(rule, local);

			if (!covered.isEmpty())
			{
				VarianceGainFunction function = new VarianceGainFunction(local);
				gain += function.evaluate(rule).getValue();
				local.remove(covered);
			}
		}

		gain /= rules.size();
		return new Unit<Double>(gain);
	}
}