/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.measure.hierarchical;

import net.sf.myra.datamining.Measure;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.Unit;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.framework.NumericCost;

/**
 * @author Fernando Esteban Barril Otero
 */
public class hFmeasureMeasure
	implements Measure
{
	public Unit<Double> evaluate(Dataset dataset, Model model)
	{
		int correct = 0;
		int predicted = 0;
		int total = 0;

		for (Instance instance : dataset)
		{
			Label pLabel = model.classify(instance);
			Label tLabel = instance.getLabel();

			// the root label is not considered as a hit
			correct += (Math.max(tLabel.intersect(pLabel) - 1, 0));
			predicted += (Math.max(pLabel.cardinality() - 1, 0));
			total += (tLabel.cardinality() - 1);
		}

		double hP = correct / (double) predicted;

		if (!NumericCost.isNumber(hP))
		{
			hP = 0.0;
		}

		double hR = correct / (double) total;

		if (!NumericCost.isNumber(hR))
		{
			hR = 0.0;
		}

		double hF = (2 * hP * hR) / (hP + hR);

		return new Unit<Double>(hF);
	}
}