/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.model;

import java.util.LinkedList;
import java.util.List;
import net.sf.myra.datamining.ProbabilisticRule;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Instance;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ProbabilisticRuleList
	extends ProbabilisticRuleModel
	implements Comparable<ProbabilisticRuleList>
{
	private static final long serialVersionUID = 450002538007785073L;

	public ProbabilisticRuleList()
	{
		super(new LinkedList<Rule>());
	}

	@Override
	public List<Rule> getRules()
	{
		return (List<Rule>) super.getRules();
	}

	/**
	 * The cost (quality) of this rule list.
	 */
	@SuppressWarnings("rawtypes")
	private Comparable confidence;

	public double[] probabilities(Instance instance)
	{
		for (Rule rule : getRules())
		{
			if (rule.isCovered(instance))
			{
				return ((ProbabilisticRule) rule).getProbabiblities();
			}
		}

		throw new IllegalArgumentException("Could not classify the instance");
	}

	/**
	 * Returns the cost of this rule list.
	 * 
	 * @return the cost of this rule list.
	 */
	@SuppressWarnings("rawtypes")
	public Comparable getConfidence()
	{
		return confidence;
	}

	/**
	 * Sets the confidence value of this rule list.
	 * 
	 * @param confidence the confidence value to set.
	 */
	@SuppressWarnings("rawtypes")
	public void setConfidence(Comparable confidence)
	{
		this.confidence = confidence;
	}

	@SuppressWarnings("unchecked")
	public int compareTo(ProbabilisticRuleList o)
	{
		int c = confidence.compareTo(o.confidence);
		return (c == 0) ? Double.compare(o.size(), size()) : c;
	}
}