/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.model;

import java.util.HashSet;
import java.util.Set;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ProbabilisticRuleSet
	extends ProbabilisticRuleModel
{
	private static final long serialVersionUID = -4445286700248535749L;

	public ProbabilisticRuleSet()
	{
		super(new HashSet<Rule>());
	}

	@Override
	public Set<Rule> getRules()
	{
		return (Set<Rule>) super.getRules();
	}

	public double[] probabilities(Instance instance)
	{
		Metadata metadata = instance.getMetadata();
		Label prototype = metadata.getLabel();

		double[] probability = new double[prototype.length()];
		int rules = 0;

		for (Rule rule : getRules())
		{
			if (!rule.isEmpty() && rule.isCovered(instance))
			{
				Label predicted = rule.getConsequent();

				for (int i = 0; i < probability.length; i++)
				{
					if (predicted.active(i))
					{
						probability[i]++;
					}
				}

				rules++;
			}
		}

		// if no rule covers the instance, use the default rule
		if (rules == 0)
		{
			Label predicted = getDefault().getConsequent();

			for (int i = 0; i < probability.length; i++)
			{
				if (predicted.active(i))
				{
					probability[i]++;
				}
			}

			rules = 1;
		}

		for (int i = 0; i < probability.length; i++)
		{
			double p = probability[i];
			probability[i] = (p > 0 ? p / rules : p);
		}

		return probability;
	}
}
