/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.statistics;

import java.util.ArrayList;

/**
 * Utility class to print the difference between two predictions file in terms
 * of the area under the curve of individual classes.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class Diff
{
	public static void main(String[] args)
		throws Exception
	{
		if (args.length < 2)
		{
			StringBuffer usage = new StringBuffer();
			usage.append("usage: java ");
			usage.append(Diff.class.getName());
			usage.append(" <predictions file1> <predictions file2>");

			System.out.println(usage.toString());
			System.exit(1);
		}

		Parser parser = new Parser();
		Information info1 = parser.read(args[0], CurveFactory.PRECISION_RECALL);
		Information info2 = parser.read(args[1], CurveFactory.PRECISION_RECALL);

		ArrayList<String> w1 = new ArrayList<String>();
		ArrayList<String> w2 = new ArrayList<String>();

		double[] v1 = new double[10];
		double[] v2 = new double[10];
		int[] levels = new int[10];

		for (String label : info1.getAvailable())
		{
			double area1 = info1.get(label).area();
			double area2 = info2.get(label).area();

			int level = label.split("/").length - 1;
			levels[level]++;
			v1[level] += area1;
			v2[level] += area2;

			if (area1 > area2)
			{
				w1.add(label);
			}
			else if (area2 > area1)
			{
				w2.add(label);
			}
		}

		System.out.println(args[0] + " (count=" + w1.size() + ")");

		//for (String label : w1)
		//{
		//	System.out.println(label);
		//}

		for (int i = 0; i < levels.length; i++)
		{
			if (levels[i] > 0)
			{
				System.out.print("level " + (i + 1) + ": ");
				System.out.println(v1[i] / levels[i]);
			}
		}

		System.out.println(args[1] + " (count=" + w2.size() + ")");

		//for (String label : w2)
		//{
		//	System.out.println(label);
		//}

		for (int i = 0; i < levels.length; i++)
		{
			if (levels[i] > 0)
			{
				System.out.print("level " + (i + 1) + ": ");
				System.out.println(v2[i] / levels[i]);
			}
		}
	}
}