/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2012 Fernando Esteban Barril Otero
 */

package net.sf.myra.datamining.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FilenameFilter;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

/**
 * @author Fernando Esteban Barril Otero
 */
public class ACDTRunner
{
	private static final String ACCURACY = "Acc. rate:";

	private static final String SIZE = "N. of leaf nodes:";

	private static final String TIME = "Time:";

	public static void main(String[] args)
		throws Exception
	{
		ArrayList<String> dataset = new ArrayList<String>();
		ArrayList<String> values = new ArrayList<String>();
		ArrayList<String> size = new ArrayList<String>();
		ArrayList<String> time = new ArrayList<String>();

		File directory = new File(args[0]);

		for (String file : directory.list(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
					return name.startsWith("00_") && name.endsWith("_tr.am");
				}
			}))
		{
			String filestem =
				file.substring(file.indexOf("00_") + 3, file.indexOf("_tr.am"));
			System.out.println("Processing " + filestem);

			double[] foldsAcc = new double[15];
			double[] foldsLen = new double[15];
			double[] foldsTim = new double[15];

			for (int i = 0; i < 15; i++)
			{
				double[] acc = new double[10];
				double[] len = new double[10];
				double[] tim = new double[10];

				for (int j = 0; j < 10; j++)
				{
					ArrayList<String> command = new ArrayList<String>();
					command.add("./acdt");
					command.add("--dataSet=" + "0" + j + "_" + filestem);

					ProcessBuilder builder = new ProcessBuilder(command);
					builder.redirectErrorStream(true);
					Process process = builder.start();

					BufferedReader reader =
						new BufferedReader(new InputStreamReader(
							process.getInputStream()));
					String line = null;

					boolean gather = true;

					while ((line = reader.readLine()) != null)
					{
						line = line.trim();

						if (gather)
						{
							if (line.startsWith(TIME))
							{
								tim[j] =
									new Double(line.substring(
										line.indexOf(TIME) + TIME.length() + 1)
										.trim());
							}
							else if (line.startsWith(ACCURACY))
							{
								acc[j] =
									100.0 * new Double(line.substring(
										line.indexOf(ACCURACY)
											+ ACCURACY.length() + 1).trim());
							}
							else if (line.startsWith(SIZE))
							{
								len[j] =
									new Double(line.substring(
										line.indexOf(SIZE) + SIZE.length() + 1)
										.trim());
								gather = false;
							}
						}
					}

					process.destroy();
					System.out.print(".");
				}

				System.out.println(mean(acc));

				foldsAcc[i] = mean(acc);
				foldsLen[i] = mean(len);
				foldsTim[i] = mean(tim);
			}

			dataset.add(filestem);
			values.add(stdev(foldsAcc));
			size.add(stdev(foldsLen));
			time.add(stdev(foldsTim));

			System.out.println(stdev(foldsAcc));
		}

		System.out.println();

		for (int i = 0; i < dataset.size(); i++)
		{
			System.out.print(dataset.get(i));
			System.out.print(",");
			System.out.print(values.get(i));
			System.out.print(",");
			System.out.print(size.get(i));
			System.out.print(",");
			System.out.println(time.get(i));
		}
	}

	/**
	 * Returns the mean and standard deviation of the specified distribution
	 * of values. The variance calculation assumes that cross-validation has
	 * been used.
	 * 
	 * @param numbers the distribution of values.
	 * 
	 * @return the mean and standard deviation of the specified distribution
	 * of values.
	 */
	public static String stdev(double[] numbers)
	{
		if (numbers.length == 0)
		{
			return "0.0 +/- 0.0";
		}

		double mean = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			mean += numbers[i];
		}

		mean /= numbers.length;
		double std = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			std += Math.pow(numbers[i] - mean, 2);
		}

		std /= (numbers.length - 1);
		std /= numbers.length;
		std = Math.sqrt(std);

		if (Double.isNaN(std) || Double.isInfinite(std))
		{
			std = 0.0;
		}

		NumberFormat formatter = new DecimalFormat("#,##0.000");
		return formatter.format(mean) + " +/- " + formatter.format(std);
	}

	public static double mean(double[] numbers)
	{
		double mean = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			mean += numbers[i];
		}

		return (mean / (double) numbers.length);
	}
}