/*
 * $Id$ 
 */

package net.sf.myra.datamining.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FilenameFilter;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class C45RulesRunner
{
	private static final String ACCURACY = "Tested";

	private static final String START_RULE_COUNT = "Final rules from tree 0:";

	private static final String RULE_TOKEN = "Rule";
	
	private static final String CLASS_TOKEN = "->";

	private static final String DEFAULT_RULE_TOKEN = "Default class:";

	public static void main(String[] args)
		throws Exception
	{
		ArrayList<Double> values = new ArrayList<Double>();
		ArrayList<Double> size = new ArrayList<Double>();
		ArrayList<Double> rules = new ArrayList<Double>();
		ArrayList<Double> terms = new ArrayList<Double>();

		File directory = new File(args[0]);

		for (String file : directory.list(new FilenameFilter()
		{
			public boolean accept(File dir, String name)
			{
				return name.startsWith("TR") && name.endsWith(".names");
			}
		}))
		{
			String filestem = args[0] + "/" + file;
			filestem = filestem.substring(0, filestem.lastIndexOf('.'));

			System.out.println("Reading file... " + filestem);

			ArrayList<String> command = new ArrayList<String>();
			command.add("./c4.5");
			command.add("-f");
			command.add(filestem);

			ProcessBuilder builder = new ProcessBuilder(command);
			builder.redirectErrorStream(true);
			Process process = builder.start();

			// waiting for c4.5
			Thread.sleep(1000);

			command.clear();
			command.add("./c4.5rules");
			command.add("-f");
			command.add(filestem);
			command.add("-u");

			builder = new ProcessBuilder(command);
			builder.redirectErrorStream(true);
			process = builder.start();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(process
					.getInputStream()));
			String line = null;
			double value = 0;

			boolean countRules = false;
			double count = 0;
			
			boolean countLength = false;
			double length = 0;

			while ((line = reader.readLine()) != null)
			{
				System.out.println(line);

				if (line.startsWith(ACCURACY))
				{
					value =
						100.0 - new Double(line.substring(
							line.indexOf("(") + 1, line.indexOf("%")).trim());
				}
				else if (line.startsWith(START_RULE_COUNT))
				{
					countRules = true;
				}
				else if (line.startsWith(RULE_TOKEN) && countRules)
				{
					countLength = true;
				}
				else if (line.contains(CLASS_TOKEN))
				{
					countLength = false;
				}
				else if (countLength)
				{
					length += 1;
				}

				if (countRules)
				{
					if (line.startsWith(RULE_TOKEN))
					{
						count++;
					}
					else if (line.startsWith(DEFAULT_RULE_TOKEN))
					{
						count++;
						countRules = false;
					}
				}

			}

			values.add(value);
			size.add(length);
			rules.add(count);
			terms.add(length / count);
		}

		for (Double v : values)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) values.toArray(new Double[0])));
		System.out.println();
		
		for (Double v : size)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) size.toArray(new Double[0])));
		System.out.println();

		for (Double v : rules)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) rules.toArray(new Double[0])));
		System.out.println();

		for (Double v : terms)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) terms.toArray(new Double[0])));
	}

	/**
	 * Returns the mean and standard deviation of the specified distribution
	 * of values. The variance calculation assumes that cross-validation has
	 * been used.
	 * 
	 * @param numbers the distribution of values.
	 * 
	 * @return the mean and standard deviation of the specified distribution
	 * of values.
	 */
	public static String stdev(Double[] numbers)
	{
		if (numbers.length == 0)
		{
			return "0.0 +/- 0.0";
		}

		double mean = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			mean += numbers[i];
		}

		mean /= numbers.length;
		double std = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			std += Math.pow(numbers[i] - mean, 2);
		}

		std /= (numbers.length - 1);
		std /= numbers.length;
		std = Math.sqrt(std);

		if (Double.isNaN(std) || Double.isInfinite(std))
		{
			std = 0.0;
		}

		NumberFormat formatter = new DecimalFormat("#,##0.000");
		return formatter.format(mean) + " +/- " + formatter.format(std);
	}
}