/*
 * $Id$ 
 */

package net.sf.myra.datamining.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FilenameFilter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class CN2Runner
{
	private static final String ACCURACY = "Overall accuracy:";

	private static final String START_RULE_COUNT = "ORDERED RULE LIST";

	private static final String RULE_TOKEN = "IF";

	private static final String TERM_TOKEN = "AND";

	private static final String DEFAULT_RULE_TOKEN = "(DEFAULT)";

	public static void main(String[] args)
		throws Exception
	{
		ArrayList<Double> values = new ArrayList<Double>();
		ArrayList<Double> size = new ArrayList<Double>();
		ArrayList<Double> rules = new ArrayList<Double>();
		ArrayList<Double> terms = new ArrayList<Double>();
		File directory = new File(args[0]);
		
		// ordered by default
		String type = "o ";
		
		if (args.length == 2 && args[1].equals("u"))
		{
			type = "u ";
		}

		for (String file : directory.list(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
					return name.startsWith("TR") && name.endsWith(".att");
				}
			}))
		{
			String fullname = new File(directory, file).getAbsolutePath();

			StringBuffer commands = new StringBuffer();
			commands.append("a ");
			commands.append(type);
			commands.append("r ");
			commands.append("a ");
			// attributes
			commands.append(fullname);
			commands.append("\n r ");
			commands.append("e ");
			// training examples
			commands.append(fullname.replace(".att", ".exs"));
			commands.append("\n i ");
			commands.append("r ");
			commands.append("e ");
			// test examples
			commands.append(fullname.replace("TR", "TS")
				.replace(".att", ".exs"));
			commands.append("\n x ");
			commands.append("a ");
			commands.append("q ");
			commands.append("q ");

			System.out.println("Reading file... " + file);

			ProcessBuilder builder = new ProcessBuilder("cn");
			builder.redirectErrorStream(true);

			Process process = builder.start();

			BufferedReader reader =
				new BufferedReader(new InputStreamReader(
					process.getInputStream()));
			PrintWriter writer =
				new PrintWriter(new BufferedWriter(new OutputStreamWriter(
					process.getOutputStream())));
			StringBuffer line = new StringBuffer();
			int c = -1;

			System.out.println(commands.toString());
			writer.write(commands.toString());
			writer.flush();

			boolean countRules = false;
			double count = 0;
			double length = 0;

			while ((c = reader.read()) != -1)
			{
				if (c != '\n')
				{
					line.append(Character.toString((char) c));
				}

				String current = line.toString();

				if (current.startsWith(ACCURACY) && c == '\n')
				{
					values.add(new Double(current.substring(
						current.indexOf(ACCURACY) + ACCURACY.length() + 1,
						current.indexOf("%")).trim()));
					break;
				}

				if (current.contains(START_RULE_COUNT) && c == '\n')
				{
					countRules = true;
				}

				if (countRules && c == '\n')
				{
					if (current.startsWith(RULE_TOKEN))
					{
						count++;
						length++;
					}
					else if (current.startsWith(DEFAULT_RULE_TOKEN))
					{
						count++;
						countRules = false;
						rules.add(count);
					}
					else if (current.trim().startsWith(TERM_TOKEN))
					{
						length++;
					}
				}

				if (c == '\n')
				{
					System.out.println(line.toString());
					line.delete(0, line.length());
				}
			}

			size.add(length);
			terms.add(length / count);
			process.destroy();
		}

		for (Double v : values)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) values.toArray(new Double[0])));
		System.out.println();

		for (Double v : size)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) size.toArray(new Double[0])));
		System.out.println();

		for (Double v : rules)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) rules.toArray(new Double[0])));
		System.out.println();

		for (Double v : terms)
		{
			System.out.println(v);
		}

		System.out.println(stdev((Double[]) terms.toArray(new Double[0])));
	}

	/**
	 * Returns the mean and standard deviation of the specified distribution
	 * of values. The variance calculation assumes that cross-validation has
	 * been used.
	 * 
	 * @param numbers the distribution of values.
	 * 
	 * @return the mean and standard deviation of the specified distribution
	 * of values.
	 */
	public static String stdev(Double[] numbers)
	{
		if (numbers.length == 0)
		{
			return "0.0 +/- 0.0";
		}

		double mean = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			mean += numbers[i];
		}

		mean /= numbers.length;
		double std = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			std += Math.pow(numbers[i] - mean, 2);
		}

		std /= (numbers.length - 1);
		std /= numbers.length;
		std = Math.sqrt(std);

		if (Double.isNaN(std) || Double.isInfinite(std))
		{
			std = 0.0;
		}

		NumberFormat formatter = new DecimalFormat("#,##0.000");
		return formatter.format(mean) + " +/- " + formatter.format(std);
	}
}