
package net.sf.myra.datamining.util;

import static net.sf.myra.datamining.Main.TR_PREFIX;
import static net.sf.myra.datamining.Main.TS_PREFIX;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.NominalAttribute;
import net.sf.myra.datamining.io.Helper;

public class MissingValueFilter
{
	public static void main(String[] args)
		throws IOException
	{
		File directory = new File(args[0]);
		String pattern = args[1];

		for (int i = 0;; i++)
		{
			String fold = (i < 10) ? "0" + i : Integer.toString(i);
			File tr = new File(directory, TR_PREFIX + fold + "_" + pattern);
			File ts = new File(directory, TS_PREFIX + fold + "_" + pattern);

			if (tr.exists() && ts.exists())
			{
				Dataset trainingSet = Helper.open(tr);
				Dataset testSet = Helper.open(ts);

				List<Instance> instances = new LinkedList<Instance>();
				instances.addAll(trainingSet.getInstances());
				instances.addAll(testSet.getInstances());

				Metadata metadata = trainingSet.getMetadata();

				for (Instance instance : instances)
				{
					for (Attribute attribute : metadata.getPredictor())
					{
						int index = metadata.getIndex(attribute.getName());

						if (attribute.isMissing(instance.value(index)))
						{
							String value = null;

							if (attribute instanceof NominalAttribute)
							{
								value =
									((NominalAttribute) attribute)
										.modal(instances);
							}
							else
							{
								value =
									Double
										.toString(((ContinuousAttribute) attribute)
											.mean(instances));
							}

							instance.set(attribute, value);
						}
					}
				}

				Helper.getHelper(tr).write(directory, trainingSet);
				Helper.getHelper(tr).write(directory, testSet);
			}
			else
			{
				break;
			}
		}
	}
}
