/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.datamining.util;

import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.ClassHierarchy;
import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.Node;
import net.sf.myra.datamining.data.NominalAttribute;
import net.sf.myra.datamining.io.Helper;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ShowDatasetDetails
{
	public static void main(String[] args)
		throws Exception
	{
		if (args.length != 1)
		{
			System.out.println("Usage: " + ShowDatasetDetails.class.getName()
				+ " <dataset file>");
			System.exit(1);
		}

		Dataset dataset = Helper.open(args[0]);
		Metadata metadata = dataset.getMetadata();

		StringBuffer output = new StringBuffer();

		output.append("=== Dataset Information ===\n\n");
		output.append("Relation: ");
		output.append(metadata.getName());
		output.append("\nInstances: ");
		output.append(dataset.getSize());
		output.append("\nAttributes: ");
		output.append(metadata.getPredictor().size());

		int cAttributes = 0;
		int nAttributes = 0;

		for (Attribute attribute : metadata.getPredictor())
		{
			if (attribute instanceof ContinuousAttribute)
			{
				cAttributes++;
			}
			else if (attribute instanceof NominalAttribute)
			{
				nAttributes++;
			}
		}

		output.append(" (");

		if (cAttributes > 0)
		{
			output.append(cAttributes);
			output.append(" continuous");
		}

		if (nAttributes > 0)
		{
			if (cAttributes > 0)
			{
				output.append(", ");
			}

			output.append(nAttributes);
			output.append(" nominal");
		}

		output.append(")");

		output.append("\nClasses: ");
		output.append(metadata.getTarget().getValues().size());
		output.append("\nDuplicates: ");
		output.append(dataset.hasDuplicates() ? "yes" : "no");
		output.append("\nHierarchical: ");
		output.append(metadata.isHierarchical() ? "yes" : "no");

		if (metadata.isHierarchical())
		{
			boolean tree = metadata.getClassHierarchy().isTree();
			output.append(tree ? " (TREE)" : " (DAG)");

			if (tree)
			{
				output.append("\nNumber of levels: ");
				output.append(metadata.getClassHierarchy().getLevels());
			}
		}

		output.append("\nMulti-Label: ");

		if (dataset.isMultilabel())
		{
			int count = 0;

			for (Instance instance : dataset)
			{
				if (instance.isMultilabel())
				{
					count++;
				}
			}

			output.append("yes (" + count + " instances)");
		}
		else
		{
			output.append("no");
		}

		output.append("\nVirtual class hierarchy: ");

		int count = 0;

		for (Instance instance : dataset)
		{
			if (!instance.isLeaf())
			{
				count++;
			}
		}

		if (count == 0)
		{
			output.append("yes");
		}
		else
		{
			output.append("no (" + count + " instances)");
		}

		output.append("\n");

		if (metadata.getClassHierarchy() != null)
		{
			output.append("\n=== Class Information ===\n");
			ClassHierarchy hierarchy = metadata.getClassHierarchy();

			for (Node node : hierarchy.flat())
			{
				output.append("\n");
				output.append(node.getLabel());
				output.append(" -> ");
				output.append(dataset.getInstances(node.getLabel()).size());
			}
		}

		System.out.print(output);
	}
}