/*
 * $Id$ 
 */

package net.sf.myra.datamining.util;

import java.util.LinkedList;
import java.util.List;

import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.util.FlatCrossValidation.Partition;
import net.sf.myra.framework.RandomGenerator;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                    #$
 */
public class ValidationHelper
{
	/**
	 * The default size of the validation set (1/10 of the dataset).  
	 */
	public static final int FRACTION = 10;

	/**
	 * The (new) training set.
	 */
	private Dataset training;

	/**
	 * The validation set.
	 */
	private Dataset validation;

	/**
	 * Creates a new <code>ValidationHelper</code>.
	 * 
	 * @param dataset the dataset to be divided.
	 */
	public ValidationHelper(Dataset dataset)
	{
		this(dataset, FRACTION);
	}

	/**
	 * Creates a new <code>ValidationHelper</code>.
	 * 
	 * @param dataset the dataset to be divided.
	 * @param fraction the fraction to be used as validation (the size of the
	 *        validation will be <code>dataset.getSize() / fraction</code>).
	 */
	public ValidationHelper(Dataset dataset, int fraction)
	{
		Partition[] partitions =
			FlatCrossValidation.split(dataset, fraction,
				FlatCrossValidation.Mode.STRATIFIED);

		int v = RandomGenerator.getInstance().nextInt(fraction);
		validation = new Dataset(dataset.getMetadata());
		validation.setInstances(partitions[v].getInstances());

		training = new Dataset(dataset.getMetadata());
		List<Instance> instances = new LinkedList<Instance>();

		for (int i = 0; i < partitions.length; i++)
		{
			if (i != v)
			{
				instances.addAll(partitions[i].getInstances());
			}
		}

		training.setInstances(instances);
	}

	/**
	 * @return the training
	 */
	public Dataset getTraining()
	{
		return training;
	}

	/**
	 * @return the validation
	 */
	public Dataset getValidation()
	{
		return validation;
	}
}