/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

import java.util.LinkedList;
import java.util.List;

/**
 * This class represents a daemon action made by the combination of several
 * daemon actions. The actions are performed in cascade order of insertion.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class CompoundDaemonAction
	implements DaemonAction
{
	/**
	 * The list of actions.
	 */
	private List<DaemonAction> actions;

	/**
	 * Default constructor.
	 */
	public CompoundDaemonAction()
	{
		actions = new LinkedList<DaemonAction>();
	}

	/**
	 * Adds the specifided action.
	 * 
	 * @param action the action to add.
	 */
	public void add(DaemonAction action)
	{
		actions.add(action);
	}

	/**
	 * Removes the specified action.
	 * 
	 * @param action the action to remove.
	 */
	public void remove(DaemonAction action)
	{
		actions.remove(action);
	}

	public void perform(Colony colony)
	{
		for (DaemonAction action : actions)
		{
			action.perform(colony);
		}
	}
}