/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

import java.util.LinkedList;
import java.util.List;

/**
 * This class represents a local search method made by the combination
 * of several local search methods. The local search methods are applied
 * in a cascade order of insertion. In other words, the input of the
 * method is the input of the previous one, with the exception of the first.
 *  
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class CompoundLocalSearch
	implements LocalSearch
{
	/**
	 * The list of local search methods.
	 */
	private List<LocalSearch> methods;

	/**
	 * Default constructor.
	 */
	public CompoundLocalSearch()
	{
		methods = new LinkedList<LocalSearch>();
	}

	/**
	 * Adds the specified local search method.
	 * 
	 * @param localSearch the local search method to add.
	 */
	public void add(LocalSearch localSearch)
	{
		methods.add(localSearch);
	}

	/**
	 * Removes the specified local search method.
	 * 
	 * @param localSearch the local search method to remove.
	 */
	public void remove(LocalSearch localSearch)
	{
		methods.remove(localSearch);
	}

	public Trail explore(Trail trail)
	{
		for (LocalSearch method : methods)
		{
			trail = method.explore(trail);
		}
		
		return trail;
	}
}