/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Default <code>Trail</code> implementation that contains no duplicate
 * vertices.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class DefaultTrail
	extends Trail
{
	private static final long serialVersionUID = -5781261957798208894L;

	/**
	 * The set of vertices.
	 */
	private Set<Vertex<?>> vertices;

	/**
	 * The last vertex added to the trail.
	 */
	private Vertex<?> last;

	/**
	 * Creates a new <code>DefaultTrail</code> instance.
	 */
	public DefaultTrail()
	{
		vertices = new LinkedHashSet<Vertex<?>>();
	}

	@Override
	public void add(Vertex<?> vertex)
	{
		vertices.add(vertex);
		last = vertex;
	}

	@Override
	public void remove(Vertex<?> vertex)
	{
		vertices.remove(vertex);

		if (vertex.equals(last))
		{
			for (Vertex<?> v : vertices)
			{
				last = v;
			}
		}

		// if the trail is changed, the cost is no longer valid
		setCost(null);
	}

	@Override
	public Collection<Vertex<?>> getVertices()
	{
		return Collections.unmodifiableSet(vertices);
	}

	@Override
	public Vertex<?> getLast()
	{
		return last;
	}

	@Override
	public boolean isEmpty()
	{
		return vertices.isEmpty();
	}

	@Override
	public boolean equals(Object o)
	{
		return equals((DefaultTrail) o);
	}

	private boolean equals(DefaultTrail o)
	{
		if (vertices.size() == o.getVertices().size())
		{
			Iterator<Vertex<?>> nodes1 = vertices.iterator();
			Iterator<Vertex<?>> nodes2 = o.getVertices().iterator();

			while (nodes1.hasNext())
			{
				if (nodes1.next().compareTo(nodes2.next()) != 0)
				{
					return false;
				}
			}

			return true;
		}

		return false;
	}

	@Override
	public DefaultTrail clone()
	{
		DefaultTrail trail = (DefaultTrail) super.clone();
		trail.vertices = new LinkedHashSet<Vertex<?>>(vertices);
		return trail;
	}
}