/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

/**
 * The colony environment (in practice, only the current colony iteration).
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class Environment
{
	/**
	 * The current colony iteration.
	 */
	private int iteration;

	/**
	 * The environment colony.
	 */
	private Colony colony;

	/**
	 * The current scheduler.
	 */
	private Scheduler scheduler;

	/**
	 * Creates a new <code>Environment</code> instance.
	 * 
	 * @param colony
	 */
	public Environment(Colony colony, Scheduler scheduler)
	{
		this.colony = colony;
		this.scheduler = scheduler;
		this.iteration = 0;
	}

	/**
	 * Returns the colony.
	 * 
	 * @return the colony.
	 */
	public Colony getColony()
	{
		return colony;
	}

	/**
	 * Returns the scheduler.
	 * 
	 * @return the scheduler.
	 */
	public Scheduler getScheduler()
	{
		return scheduler;
	}

	/**
	 * Returns the current iteration of the colony.
	 * 
	 * @return the colony's current iteration.
	 */
	public int getIteration()
	{
		return iteration;
	}

	/**
	 * Increment the colony's iteration value.
	 */
	public void increment()
	{
		iteration++;
	}
}