/*
 * $Id$ 
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================ 
 */

package net.sf.myra.framework;

import java.io.Serializable;

/**
 * This interface represents additional information that can be added to the
 * vertex or edge.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public interface Info
	extends Cloneable, Serializable
{
	/**
	 * Returns a copy of this <code>Info</code> object.
	 * 
	 * @return a copy of this <code>Info</code> object.
	 */
	public Info clone();

	/**
	 * This class represents an <code>Info</code> which has no information
	 * associated.
	 */
	public static class Void
		implements Info
	{
		private static final long serialVersionUID = -7320050886800502139L;

		public Void clone()
		{
			try
			{
				return (Void) super.clone();
			}
			catch (CloneNotSupportedException e)
			{
				throw new UnknownError(getClass() + ": clone()");
			}
		}
	}
}