/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

/**
 * Default numeric cost implementation. Note that this implementation assumes
 * that bigger values are better than smaller values.
 *  
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class NumericCost
	implements Cost
{
	private static final long serialVersionUID = 7862131730997730696L;

	/**
	 * The numeric (double) value.
	 */
	private double value;

	/**
	 * Default constructor.
	 * 
	 * @param value the cost value.
	 */
	public NumericCost(double value)
	{
		if (Double.isNaN(value) || Double.isInfinite(value))
		{
			throw new IllegalArgumentException("Invalid value: " + value);
		}

		this.value = value;
	}

	/**
	 * Returns the cost value.
	 * 
	 * @return the cost value.
	 */
	public double getValue()
	{
		return value;
	}

	public int compareTo(Cost o)
	{
		return compareTo((NumericCost) o);
	}

	public int compareTo(NumericCost o)
	{
		return Double.compare(value, o.value);
	}

	@Override
	public NumericCost clone()
	{
		try
		{
			return (NumericCost) super.clone();
		}
		catch (CloneNotSupportedException e)
		{
			throw new UnknownError(getClass() + ": clone()");
		}
	}

	/**
	 * Tests if the specified value is a number.
	 * 
	 * @param value the value to be tested.
	 * 
	 * @return <code>true</code> if the value is a number; <code>false</code>
	 *         otherwise.
	 */
	public static boolean isNumber(double value)
	{
		return !Double.isNaN(value) && !Double.isInfinite(value);
	}

	@Override
	public String toString()
	{
		return Double.toString(value);
	}
}