/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * Utility class to create class instances.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date: 2008-02-08 15:54:20#$
 */
public abstract class ObjectFactory
{
	/**
	 * Creates a new object instance of the specified class.
	 * 
	 * @param className the fully qualified class name of the object to be 
	 *        created.
	 * 
	 * @throws IllegalArgumentException if there is any error instantiating an 
	 *         object of the specified class.
	 */
	public static Object create(String className)
	{
		try
		{
			Class<?> c = Class.forName(className);
			return c.newInstance();
		}
		catch (ClassNotFoundException e)
		{
			throw new IllegalArgumentException("Class '" + className
				+ "' could not be found", e);
		}
		catch (InstantiationException e)
		{
			throw new IllegalArgumentException(className
				+ " could not be instantiated", e);
		}
		catch (IllegalAccessException e)
		{
			throw new IllegalArgumentException(className
				+ " could not be instantiated", e);
		}
		catch (NullPointerException e)
		{
			throw new IllegalArgumentException(
				"Class name not specified: null", e);
		}
	}

	/**
	 * Creates a new object instance of the specified class.
	 * 
	 * @param className the fully qualified class name of the object to be 
	 *        created.
	 * 
	 * @throws IllegalArgumentException if there is any error instantiating an 
	 *         object of the specified class.
	 */
	public static Object create(String className, Class<?>[] types,
			Object[] parameters)
	{
		try
		{
			Class<?> c = Class.forName(className);
			Constructor<?> constructor = c.getConstructor(types);

			return constructor.newInstance(parameters);
		}
		catch (ClassNotFoundException e)
		{
			throw new IllegalArgumentException("Class '" + className
				+ "' could not be found", e);
		}
		catch (NoSuchMethodException e)
		{
			throw new IllegalArgumentException("Constructor for '" + className
				+ "' could not be found", e);
		}
		catch (InstantiationException e)
		{
			throw new IllegalArgumentException(className
				+ "could not be instantiated", e);
		}
		catch (InvocationTargetException e)
		{
			throw new IllegalArgumentException(className
				+ "could not be instantiated", e);
		}
		catch (IllegalAccessException e)
		{
			throw new IllegalArgumentException(className
				+ "could not be instantiated", e);
		}
		catch (NullPointerException e)
		{
			throw new IllegalArgumentException(
				"Class name not specified: null", e);
		}
	}
}