/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

import java.io.Serializable;

/**
 * This class represents a vertex of the problem construction graph.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class Vertex<V extends Info>
	extends Graph.Element
	implements Comparable<Vertex<?>>, Cloneable, Serializable
{
	private static final long serialVersionUID = -3009776220264257396L;

	/**
	 * The vertex label.
	 */
	private String label;

	/**
	 * The vertex info object.
	 */
	private V info;

	/**
	 * The vertex's index in the graph.
	 */
	private int index;

	/**
	 * Default constructor.
	 * 
	 * @param label the vertex label.
	 */
	public Vertex(String label)
	{
		this.label = label;
	}

	/**
	 * Returns the vertex's index.
	 * 
	 * @return the vertex's index.
	 */
	int getIndex()
	{
		return index;
	}

	/**
	 * Sets the vertex's index.
	 * 
	 * @param index the vertex's index to set.
	 */
	void setIndex(int index)
	{
		this.index = index;
	}

	/**
	 * Returns the vertex label.
	 * 
	 * @return the vertex label.
	 */
	public String getLabel()
	{
		return label;
	}

	/**
	 * Sets the vertex label.
	 * 
	 * @param label the label to set.
	 */
	public void setLabel(String label)
	{
		this.label = label;
	}

	@Override
	public boolean equals(Object v)
	{
		return equals((Vertex<?>) v);
	}

	/**
	 * Compares the specified vertex with this vertex for equality. Returns
	 * <code>true</code> if the two instances have the same label.
	 *  
	 * @param v the vertex to be compared for equality with this vertex.
	 * 
	 * @return <code>true</code> if the specified vertex is equal to this
	 *         vertex; <code>false</code> otherwise.
	 */
	public boolean equals(Vertex<?> v)
	{
		return compareTo((Vertex<?>) v) == 0;
	}

	/**
	 * Returns the vertex info object.
	 * 
	 * @return the vertex info object.
	 */
	public V getInfo()
	{
		return info;
	}

	/**
	 * Sets the vertex info.
	 * 
	 * @param v the info object.
	 */
	public void setInfo(V v)
	{
		this.info = v;
	}

	@Override
	public int hashCode()
	{
		return label.hashCode();
	}

	@Override
	public String toString()
	{
		return "vertex(" + label + ") " + super.toString();
	}

	public int compareTo(Vertex<?> o)
	{
		return label.compareTo(o.label);
	}

	@Override
	@SuppressWarnings("unchecked")
	public Vertex<V> clone()
	{
		Vertex<V> vertex = (Vertex<V>) super.clone();
		vertex.info = (V) info.clone();

		return vertex;
	}
}