/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

/**
 * A class implements the <code>Weighable</code> interface to indicate that
 * it can have pheromone and heuristic information values.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public interface Weighable
{
	/**
	 * Returns the heuristic value associated with this component.
	 * 
	 * @return the heuristic value associated with this component.
	 * 
	 * @see #getHeuristicValue(int)
	 */
	public HeuristicValue getHeuristicValue();
	
	/**
	 * Returns the heuristic value associated with this component.
	 * 
	 * @param level the level of the heuristic value.
	 * 
	 * @return the heuristic value associated with this component.
	 */
	public HeuristicValue getHeuristicValue(int level);

	/**
	 * Sets the heuristic value associated with this component.
	 * 
	 * @param heuristicValue the value to be set.
	 * 
	 * @see #setHeuristicValue(HeuristicValue, int)
	 */
	public void setHeuristicValue(HeuristicValue heuristicValue);
	
	/**
	 * Sets the heuristic value associated with this component.
	 * 
	 * @param heuristicValue the value to be set.
	 * @param level the level of the heuristic value.
	 */
	public void setHeuristicValue(HeuristicValue heuristicValue, int level);

	/**
	 * Returns the pheromone value associated with this component. This is
	 * equivalent of setting the pheromone at level <code>0</code>.
	 * 
	 * @return the pheromone value associated with this component.
	 * 
	 * @see #getPheromone(int)
	 */
	public Pheromone getPheromone();
	
	/**
	 * Returns the pheromone value associated with this component at the
	 * specified level.
	 * 
	 * @param level the level of the pheromone.
	 * 
	 * @return the pheromone value associated with this component.
	 */
	public Pheromone getPheromone(int level);

	/**
	 * Sets the pheromone value associated with this component. This is
	 * equivalent of setting the pheromone at level <code>0</code>.
	 * 
	 * @param pheromone the value to be set.
	 * 
	 * @see #setPheromone(Pheromone, int)
	 */
	public void setPheromone(Pheromone pheromone);
	
	/**
	 * Sets the pheromone value associated with this component at the
	 * specified level.
	 * 
	 * @param level the level of the pheromone.
	 * @param pheromone the value to be set.
	 */
	public void setPheromone(Pheromone pheromone, int level);
}