/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.framework;

/**
 * This class represents a weight value, which can be associated to a
 * <code>WeightedComponent</code>.
 * 
 * @see Weighable
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
class Weight
	implements Cloneable
{
	/**
	 * The weight value.
	 */
	private double value;

	/**
	 * Creates a new weight with the specified value.
	 * 
	 * @param value the weight value.
	 */
	Weight(double value)
	{
		if (Double.isNaN(value) || Double.isInfinite(value))
		{
			throw new IllegalArgumentException("Invalid value: " + value);
		}

		this.value = value;
	}

	/**
	 * Returns the value of the weight.
	 * 
	 * @return the value of the weight.
	 */
	public double getValue()
	{
		return value;
	}

	/**
	 * Sets the weight value.
	 * 
	 * @param value the value to set.
	 */
	public void setValue(double value)
	{
		this.value = value;
	}

	/**
	 * Creates and returns a copy of this weight instance.
	 *
	 * @return a copy of this weight instance.
	 */
	@Override
	protected Object clone()
	{
		try
		{
			return (Weight) super.clone();
		}
		catch (CloneNotSupportedException e)
		{
			throw new UnknownError(getClass() + ": clone()");
		}
	}

	@Override
	public String toString()
	{
		return Double.toString(value);
	}
}