/*
 * $Id$ 
 */

package net.sf.myra.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.LayoutManager;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPanel;
import javax.swing.RootPaneContainer;
import javax.swing.SwingUtilities;

/**
 * Base class for panels.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
abstract class AbstractToolPanel
	extends JPanel
{
	private static final long serialVersionUID = 4195171638849961783L;

	/**
	 * Default constructor.
	 */
	AbstractToolPanel(LayoutManager manager)
	{
		super(manager);
	}

	/**
	 * A default focus adapter for the dialog glass pane.
	 */
	private static final FocusAdapter FOCUS_ADAPTER = new FocusAdapter()
	{
		@Override
		public void focusGained(FocusEvent e)
		{
			Component c = (Component) e.getSource();
			c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}
	};

	/**
	 * A default mouse adapter for the dialog glass pane.
	 */
	private final static MouseAdapter MOUSE_ADAPTER = new MouseAdapter()
	{
		@Override
		public void mouseEntered(MouseEvent e)
		{
			Component c = (Component) e.getSource();
			c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}

		@Override
		public void mouseExited(MouseEvent e)
		{
			Component c = (Component) e.getSource();
			c.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	};

	/**
	 * Shows a busy cursor.
	 */
	protected void startWaitCursor()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				Container ancestor =
					AbstractToolPanel.this.getTopLevelAncestor();
				RootPaneContainer root = (RootPaneContainer) ancestor;
				Component c = root.getGlassPane();

				c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				c.addMouseListener(MOUSE_ADAPTER);
				c.addFocusListener(FOCUS_ADAPTER);
				c.setVisible(true);
			}
		});
	}

	/**
	 * Restores the default cursor.
	 */
	protected void stopWaitCursor()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				Container ancestor =
					AbstractToolPanel.this.getTopLevelAncestor();
				RootPaneContainer root = (RootPaneContainer) ancestor;
				Component c = root.getGlassPane();

				c.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				c.removeMouseListener(MOUSE_ADAPTER);
				c.removeFocusListener(FOCUS_ADAPTER);
				c.setVisible(false);
			}
		});
	}
}
