/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui;

import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import javax.imageio.ImageIO;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * This class represents GUI entry point.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class Main
{
	/**
	 * Are we running on a OSX environment?
	 */
	public static final boolean MAC_OS_X =
		(System.getProperty("os.name").toLowerCase().startsWith("mac os x"));

	// sets OSX specific properties
	static
	{
		if (MAC_OS_X)
		{
			System.setProperty(
				"com.apple.mrj.application.apple.menu.about.name", "Myra");

			try
			{
				Object application =
					Class.forName("com.apple.eawt.Application").getConstructor(
						(Class[]) null).newInstance((Object[]) null);

				Method method =
					application.getClass().getDeclaredMethod(
						"removeAboutMenuItem");
				method.invoke(application);

				method =
					application.getClass().getDeclaredMethod(
						"setDockIconImage", Image.class);

				method.invoke(application, ImageIO.read(application.getClass()
					.getResourceAsStream("/myra-icon-108.png")));
			}
			catch (Exception e)
			{
				// it shouldn't happend, but we are not bothered, since they
				// are cosmetic settings
			}
		}
	}

	/**
	 * Display the GUI window.
	 * 
	 * @param args command-line arguments.
	 */
	public static void main(String[] args)
		throws Exception
	{
		String argument = null;

		// is the vm argument specified?
		if (args.length > 1)
		{
			for (int i = 0; i < args.length; i++)
			{
				if ("-vm".equals(args[i]) && (i + 1) < args.length)
				{
					argument = args[i + 1];
				}
			}
		}

		if (argument == null)
		{
			argument =
				System.getProperty("java.home") + File.separator + "bin"
					+ File.separator + "java";
		}

		try
		{
			Process process = new ProcessBuilder(argument, "-version").start();

			if (process.waitFor() != 0)
			{
				throw new IllegalArgumentException();
			}
		}
		// in case of an exception, try to use the default java executable
		// available on the system path
		catch (Exception e)
		{
			argument = "java";
		}

		final String vm = argument;

		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					UIManager.setLookAndFeel(UIManager
						.getSystemLookAndFeelClassName());
				}
				catch (Exception e)
				{
					// quietly ignored (the default will be used)
				}

				Frame frame = new Frame(vm);

				if (!MAC_OS_X)
				{
					try
					{
						frame.setIconImage(ImageIO.read(getClass()
							.getResourceAsStream("/myra-icon-48.png")));
					}
					catch (IOException e)
					{
						// should not happen, but in the worst case the default
						// icon will be used
					}
				}

				frame.setVisible(true);
			}
		});
	}
}