/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import net.sf.myra.gui.parser.AccuracyParser;
import net.sf.myra.gui.parser.ResultParser;
import net.sf.myra.gui.parser.SizeParser;
import net.sf.myra.gui.parser.hFmeasureParser;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ParserPanel
	extends AbstractToolPanel
{
	private static final long serialVersionUID = -484622103505571880L;

	/**
	 * The value to be parsed.
	 */
	private String text;

	/**
	 * The text field to display the result of the parser.
	 */
	private JTextArea result;

	/**
	 * The result parser options.
	 */
	@SuppressWarnings("rawtypes")
	private JComboBox parsers;

	/**
	 * The external file text field.
	 */
	private JTextField externalFile;

	/**
	 * Default constructor.
	 * 
	 * @param frame
	 *            the parent frame.
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public ParserPanel(Frame frame)
	{
		super(new BorderLayout());
		this.text = frame.getOutput();

		ResultParser[] available =
			{ new AccuracyParser(), new SizeParser(), new hFmeasureParser() };
		parsers = new JComboBox(available);

		// parser options

		JPanel options = new JPanel(new BorderLayout());
		options.setBorder(BorderFactory.createTitledBorder(" Parser "));

		options.add(parsers, BorderLayout.CENTER);

		JButton button = new JButton("Show");
		button.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					final ResultParser p =
						(ResultParser) parsers.getSelectedItem();
					final String filename = externalFile.getText().trim();

					if (!"".equals(filename))
					{
						new Thread()
							{
								public void run()
								{
									startWaitCursor();
									setResult("parsing...");
									BufferedReader reader = null;

									try
									{
										StringBuffer buffer =
											new StringBuffer();
										reader =
											new BufferedReader(new FileReader(
												filename));
										String line = null;

										while ((line = reader.readLine()) != null)
										{
											buffer.append(line);
											buffer.append("\n");
										}

										setResult(p.parse(buffer.toString()));
									}
									catch (IOException exception)
									{
										JOptionPane.showMessageDialog(
											ParserPanel.this,
											exception.getMessage(),
											"Error reading external file",
											JOptionPane.ERROR_MESSAGE);
									}
									catch (Exception e)
									{
										setResult(e.toString());
									}
									finally
									{
										if (reader != null)
										{
											try
											{
												reader.close();
											}
											catch (IOException e)
											{
												throw new RuntimeException(e);
											}
										}

										stopWaitCursor();
									}
								}
							}.start();
					}
					else if (!"".equals(text.trim()))
					{
						new Thread()
							{
								public void run()
								{
									startWaitCursor();
									setResult("parsing...");

									try
									{
										setResult(p.parse(text));
									}
									catch (Exception e)
									{
										setResult(e.toString());
									}
									finally
									{
										stopWaitCursor();
									}
								}
							}.start();
					}
					else
					{
						JOptionPane.showMessageDialog(ParserPanel.this,
							"Both output and external file are empty.",
							"Empty input", JOptionPane.INFORMATION_MESSAGE);
					}
				}
			});
		options.add(button, BorderLayout.EAST);

		add(options, BorderLayout.NORTH);

		// result area

		result = new JTextArea();
		result.setEditable(false);
		JScrollPane output = new JScrollPane(result);

		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(BorderFactory.createEmptyBorder(0, 3, 3, 3));
		panel.add(output, BorderLayout.CENTER);
		add(panel, BorderLayout.CENTER);

		// external file

		JPanel external = new JPanel(new BorderLayout());
		external.setBorder(BorderFactory.createTitledBorder(" External File "));

		button = new JButton("File...");
		button.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					Frame.CHOOSER.setFileSelectionMode(JFileChooser.FILES_ONLY);
					int c = Frame.CHOOSER.showOpenDialog(getParent());

					if (c == JFileChooser.APPROVE_OPTION)
					{
						externalFile.setText(Frame.CHOOSER.getSelectedFile()
							.getAbsolutePath());
					}
				}
			});
		external.add(button, BorderLayout.WEST);

		externalFile = new JTextField();
		externalFile.setEditable(false);
		external.add(externalFile, BorderLayout.CENTER);

		button = new JButton("Clear");
		button.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					result.setText("");
					externalFile.setText("");
				}
			});
		external.add(button, BorderLayout.EAST);

		add(external, BorderLayout.SOUTH);
	}

	/**
	 * Updates the result field value.
	 * 
	 * @param text
	 *            the new value.
	 */
	private void setResult(final String text)
	{
		SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					result.setText(text);
				}
			});
	}
}