/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

/**
 * This class represents the parser for the <code>AccuracyMeasure</code>
 * evaluation measure.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class AccuracyParser
	extends ResultParser
{
	private final static String TOKEN = "(AccuracyMeasure): ";

	@Override
	public String parse(String text)
	{
		ArrayList<Double> values = new ArrayList<Double>();

		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		try
		{
			while ((line = input.readLine()) != null)
			{
				int start = line.indexOf(TOKEN);

				if (start != -1)
				{
					start += TOKEN.length();
					int end = line.indexOf(' ', start);

					if (end == -1)
					{
						values.add(new Double(line.substring(start)) * 100);
					}
					else
					{
						values
							.add(new Double(line.substring(start, end)) * 100);
					}
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		StringBuffer output = new StringBuffer();
		output.append("=== Accuracy Parser ===\n");

		for (Double v : values)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) values.toArray(new Double[0])));

		return output.toString();
	}

	public static void main(String[] args)
		throws Exception
	{
		String dir = args[0];
		
		for (int i = 1; i < args.length; i++)
		{
			print(dir, args[i]);
		}
	}

	public static void print(final String dir, final String pattern)
		throws Exception
	{
		File directory = new File(dir);

		ArrayList<Double> values = new ArrayList<Double>();
		AccuracyParser parser = new AccuracyParser();

		for (String file : directory.list(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
					//return name.startsWith(pattern) && name.endsWith(".out");
					return name.matches(pattern + "-\\d\\d.out");
				}
			}))
		{
			StringBuffer buffer = new StringBuffer();
			BufferedReader reader =
				new BufferedReader(new FileReader(new File(directory, file)));
			String line = null;

			while ((line = reader.readLine()) != null)
			{
				buffer.append(line);
				buffer.append("\n");
			}

			reader =
				new BufferedReader(new StringReader(parser.parse(buffer
					.toString())));
			line = null;
			String last = null;

			while ((line = reader.readLine()) != null)
			{
				last = line;
			}

			Double v = new Double(last.substring(0, last.indexOf(' ')));

			if (v > 0)
			{
				values.add(v);
			}

		}

		//for (Double v : values)
		//{
			//System.out.println(v);
		//}

		//System.out.println();
		System.out.println(parser.stdev((Double[]) values
			.toArray(new Double[0])));
		//System.out.println("(" + values.size() + ")");
	}
}