/*
 * $Id$ 
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;
import java.util.TreeSet;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class BatchUtil
{
	private static final ResultParser[] PARSERS =
		{ new AccuracyParser(), new SizeParser() };

	private static final ResultParser ELAPSED_TIME = new ElapsedTimeParser();

	public static void main(String[] args)
		throws Exception
	{
		if (args.length != 1)
		{
			System.out.println("usage: " + BatchUtil.class.getName()
				+ " <directory>");
			System.exit(1);
		}

		File directory = new File(args[0]);

		if (!directory.isDirectory())
		{
			System.out.println("Directory not found: " + args[0]);
			System.exit(1);
		}

		File[] files = directory.listFiles();
		Map<String, TreeSet<String>> mapping =
			new TreeMap<String, TreeSet<String>>();

		for (File file : files)
		{
			String name = file.getName();
			
			if (name.startsWith("crx")
					|| name.startsWith("ecoli")
					|| name.startsWith("tic-tac-toe")
					|| name.startsWith("vehicle")
					|| name.startsWith("zoo"))
			{
				continue;
			}

			int start = name.lastIndexOf('-');
			int end = -1;

			if (start != -1)
			{
				end = name.indexOf('.', start);
			}

			String group = "";

			if (start != -1 && end != -1)
			{
				group = name.substring(start, end);
			}

			BufferedReader reader = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text = null;

			while ((text = reader.readLine()) != null)
			{
				buffer.append(text);
				buffer.append("\n");
			}
			
			reader.close();

			text = buffer.toString();
			buffer = new StringBuffer();
			buffer.append(name);

			for (ResultParser parser : PARSERS)
			{
				for (String result : parse(parser.parse(text)))
				{
					buffer.append(",");
					buffer.append(result);
				}
			}

			buffer.append(",");
			buffer.append(ELAPSED_TIME.parse(text));

			TreeSet<String> set = mapping.get(group);

			if (set == null)
			{
				set = new TreeSet<String>();
				mapping.put(group, set);
			}

			set.add(buffer.toString());
		}

		for (String key : mapping.keySet())
		{
			double total = 0.0;
			System.out.println(key);

			for (String line : mapping.get(key))
			{
				System.out.println(line);
				total +=
					(Double.parseDouble(line.substring(line.indexOf(',') + 1,
						line.indexOf(' '))));
			}

			System.out.println(total / mapping.get(key).size());
			System.out.println();
		}
	}

	private static String[] parse(String result)
		throws IOException
	{
		BufferedReader reader = new BufferedReader(new StringReader(result));
		String line = null;
		ArrayList<String> lines = new ArrayList<String>();

		while ((line = reader.readLine()) != null)
		{
			if (line.contains("+/-"))
			{
				lines.add(line);
			}
		}

		return lines.toArray(new String[0]);
	}

	static class ElapsedTimeParser
		extends ResultParser
	{
		private final static String TOKEN = "Elapsed time: ";

		@Override
		public String parse(String text)
		{
			BufferedReader input = new BufferedReader(new StringReader(text));
			String line = null;

			try
			{
				while ((line = input.readLine()) != null)
				{
					int start = line.indexOf(TOKEN);

					if (start != -1)
					{
						start += TOKEN.length();
						int end = line.indexOf(' ', start);

						return line.substring(start, end);
					}
				}
			}
			catch (IOException e)
			{
				throw new RuntimeException(e);
			}

			return "<unspecified elapsed time>";
		}

	}
}