/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

/**
 * This class represents the parser for the elapsed time of a run.
 * 
 * @author Fernando Esteban Barril Otero
 */
public class ElapsedTimeParser
	extends ResultParser
{
	private final static String TOKEN = "Elapsed time: ";

	/**
	 * Token used to identify if the output file contains more than one run.
	 */
	private final static String MULTIPLE = "Measure";

	@Override
	public String parse(String text)
	{
		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		int count = 0;
		double value = 0;

		try
		{
			while ((line = input.readLine()) != null)
			{
				int start = line.indexOf(TOKEN);

				if (start != -1)
				{
					start += TOKEN.length();
					int end = line.indexOf(' ', start);
					value = new Double(line.substring(start, end));
				}
				else if (line.startsWith(MULTIPLE))
				{
					count++;
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		StringBuffer output = new StringBuffer();
		output.append("=== Elapsed Time Parser ===\n\n");

		output.append(value / count);
		output.append(" (average over ");
		output.append(count);
		output.append(" executions)");

		return output.toString();
	}

	public static void main(String[] args)
		throws Exception
	{
		final String pattern = args[1];
		File directory = new File(args[0]);

		ArrayList<Double> values = new ArrayList<Double>();
		ElapsedTimeParser parser = new ElapsedTimeParser();

		for (String file : directory.list(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
					return name.startsWith(pattern) && name.endsWith(".out");
				}
			}))
		{
			StringBuffer buffer = new StringBuffer();
			BufferedReader reader =
				new BufferedReader(new FileReader(new File(directory, file)));
			String line = null;

			while ((line = reader.readLine()) != null)
			{
				buffer.append(line);
				buffer.append("\n");
			}

			reader =
				new BufferedReader(new StringReader(parser.parse(buffer
					.toString())));
			line = null;
			String last = null;

			while ((line = reader.readLine()) != null)
			{
				last = line;
			}

			Double v = new Double(last.substring(0, last.indexOf(' ')));

			if (v > 0)
			{
				values.add(v);
			}

		}

		for (Double v : values)
		{
			System.out.println(v);
		}

		System.out.println();
		System.out.println(parser.stdev((Double[]) values
			.toArray(new Double[0])));
		System.out.println("(" + values.size() + ")");
	}
}