/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui.parser;

import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 * Base class for result parsers.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public abstract class ResultParser
{
	/**
	 * Returns the result (as a string) of parsing the specified text.
	 * 
	 * @param text the text to be parsed.
	 * 
	 * @return the result (as a string) of parsing the specified text.
	 */
	public abstract String parse(String text);

	@Override
	public String toString()
	{
		return getClass().getSimpleName();
	}

	/**
	 * Returns the mean and standard deviation of the specified distribution
	 * of values. The variance calculation assumes that cross-validation has
	 * been used.
	 * 
	 * @param numbers the distribution of values.
	 * 
	 * @return the mean and standard deviation of the specified distribution
	 * of values.
	 */
	public String stdev(Double[] numbers)
	{
		if (numbers.length == 0)
		{
			return "0.0 +/- 0.0";
		}

		double mean = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			mean += numbers[i];
		}

		mean /= numbers.length;
		double std = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			std += Math.pow(numbers[i] - mean, 2);
		}

		std /= (numbers.length - 1);
		std /= numbers.length;
		std = Math.sqrt(std);

		if (Double.isNaN(std) || Double.isInfinite(std))
		{
			std = 0.0;
		}

		NumberFormat formatter = new DecimalFormat("#,##0.000");
		return formatter.format(mean) + " +/- " + formatter.format(std);
	}
}