/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

/**
 * This class represents the parser for the
 * <code>SensitivitySpecificityMeasure</code> evaluation measure.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class SensitivitySpecificityParser
	extends ResultParser
{
	private final static String TOKEN = "): ";

	@Override
	public String parse(String text)
	{
		ArrayList<Double> sensitivity = new ArrayList<Double>();
		ArrayList<Double> specificity = new ArrayList<Double>();
		ArrayList<Double> values = new ArrayList<Double>();

		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		try
		{
			while ((line = input.readLine()) != null)
			{
				int start = line.indexOf(TOKEN);

				if (start != -1)
				{
					int end = line.indexOf("*");

					sensitivity.add(new Double(line.substring(start + 3, end)
						.trim()));

					start = end + 1;
					end = line.indexOf("=");

					specificity.add(new Double(line.substring(start, end)
						.trim()));

					start = end + 1;

					values.add(new Double(line.substring(start)));
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		StringBuffer output = new StringBuffer();
		output.append("[Sensitivity]\n");

		for (Double v : sensitivity)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) sensitivity.toArray(new Double[0])));

		output.append("\n\n[Specificity]\n");

		for (Double v : specificity)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) specificity.toArray(new Double[0])));

		output.append("\n\n[Sensitivity * Specificity]\n");

		for (Double v : values)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) values.toArray(new Double[0])));

		return output.toString();
	}
}