/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

/**
 * This class represents the parser for the rule list size.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class TreeSizeParser
	extends ResultParser
{
	//private final static String START_RULE_LIST = "=== Discovered Rules ===";

	//private final static String START_RULE = "IF";

	// constants for the bulk option

	private final static String SIZE_TOKEN = "Model size: ";

	private final static String LEAVES_TOKEN = "Number of leaf nodes: ";

	@Override
	public String parse(String text)
	{
		//return text.contains(START_RULE_LIST) ? parseList(text)
		//	: parseBulk(text);
		return parseBulk(text);
	}

	/*
	private String parseList(String text)
	{
		ArrayList<Double> size = new ArrayList<Double>();
		ArrayList<Double> terms = new ArrayList<Double>();
		ArrayList<Double> rules = new ArrayList<Double>();

		RuleParser parser = new RuleParser();
		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		try
		{
			ArrayList<String> current = null;

			while ((line = input.readLine()) != null)
			{
				if (line.indexOf(START_RULE_LIST) != -1)
				{
					current = new ArrayList<String>();
				}
				else if (line.indexOf(START_RULE) != -1)
				{
					current.add(line);
				}
				// do we completed the previous list of rules?
				else if (current != null && !current.isEmpty())
				{
					double length = 0.0;

					for (String r : current)
					{
						length += parser.parse(r).getSize();
					}

					size.add(length);
					terms.add(length / current.size());
					rules.add((double) current.size());
					
					current = null;
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		return generateOutput(size, terms, rules);
	}
	*/
	private String parseBulk(String text)
	{
		ArrayList<Double> size = new ArrayList<Double>();
		ArrayList<Double> leaves = new ArrayList<Double>();

		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		try
		{
			while ((line = input.readLine()) != null)
			{
				if (line.indexOf(SIZE_TOKEN) != -1)
				{
					int start = line.indexOf(SIZE_TOKEN) + SIZE_TOKEN.length();
					int end = line.indexOf(' ', start);

					if (end == -1)
					{
						size.add(new Double(line.substring(start)));
					}
					else
					{
						size.add(new Double(line.substring(start, end)));
					}
				}
				else if (line.indexOf(LEAVES_TOKEN) != -1)
				{
					int start =
						line.indexOf(LEAVES_TOKEN) + LEAVES_TOKEN.length();
					int end = line.indexOf(' ', start);

					if (end == -1)
					{
						leaves.add(new Double(line.substring(start)));
					}
					else
					{
						leaves.add(new Double(line.substring(start, end)));
					}
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		return generateOutput(size, leaves);
	}

	private String generateOutput(ArrayList<Double> size,
			ArrayList<Double> leaves)
	{
		StringBuffer output = new StringBuffer();
		output.append("=== Size Parser ===\n");

		output.append("\n[model size]\n");
		for (Double v : size)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) size.toArray(new Double[0])));

		output.append("\n\n[leaves per rule]\n");
		for (Double v : leaves)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) leaves.toArray(new Double[0])));

		return output.toString();
	}
}