/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.gui.parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This class represents the parser for the <code>HierarchicalMeasure</code>
 * evaluation measure.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class hFmeasureParser
	extends ResultParser
{
	private final static String START = "): {";
	private final static String END = "}";

	@Override
	public String parse(String text)
	{
		ArrayList<Double> hP = new ArrayList<Double>();
		ArrayList<Double> hR = new ArrayList<Double>();
		ArrayList<Double> hF = new ArrayList<Double>();

		BufferedReader input = new BufferedReader(new StringReader(text));
		String line = null;

		try
		{
			while ((line = input.readLine()) != null)
			{
				int start = line.indexOf(START);

				if (start != -1)
				{
					start += START.length();
					StringTokenizer values =
						new StringTokenizer(line.substring(start,
							line.indexOf(END)), ",");

					String v = values.nextToken();
					hP.add(new Double(v.substring(v.indexOf('=') + 1).trim()));

					v = values.nextToken();
					hR.add(new Double(v.substring(v.indexOf('=') + 1).trim()));

					v = values.nextToken();
					hF.add(new Double(v.substring(v.indexOf('=') + 1).trim()));
				}
			}
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		StringBuffer output = new StringBuffer();
		output.append("[hPrecision]\n");

		for (Double v : hP)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) hP.toArray(new Double[0])));

		output.append("\n\n[hRecall]\n");

		for (Double v : hR)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) hR.toArray(new Double[0])));

		output.append("\n\n[hFmeasure]\n");

		for (Double v : hF)
		{
			output.append(v);
			output.append('\n');
		}

		output.append(stdev((Double[]) hF.toArray(new Double[0])));

		return output.toString();
	}
}