/*
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2011 Fernando Esteban Barril Otero
 */

package net.sf.myra.gui.util;

/**
 * This class represents the result of a configuration on a given instance.
 *
 * @see Configuration
 */
public class Observation
	implements Comparable<Observation>
{
	/**
	 * The rank of this observation when compared againt all other observations
	 * on the same instance.
	 */
	private double rank;

	/**
	 * The perfomance achieved by the configuration.
	 */
	private double value;

	/**
	 * Creates a new <code>Observation</code>.
	 * 
	 * @param value the performance achieved by the configuration.
	 */
	public Observation(double value)
	{
		this.value = value;
	}

	/**
	 * Compares this observation's value to the specified observation's value.
	 * The comparison respects a descendent order.
	 * 
	 * @param o the observation to be compared.
	 * 
	 * @return <code>0</code> if both values are equal; negative integer if
	 *         this observation's value is greater than the specified
	 *         observation's value ; and positive integer if this
	 *         configuration's value is less than the specified observation's
	 *         value.
	 */
	public int compareTo(Observation o)
	{
		return Double.compare(o.value, value);
	}

	/**
	 * Returns the rank of this observation.
	 * 
	 * @return the rank of this observation.
	 */
	public double getRank()
	{
		return rank;
	}

	/**
	 * Sets the rank of this observation.
	 * 
	 * @param rank the rank to set.
	 */
	public void setRank(double rank)
	{
		this.rank = rank;
	}

	/**
	 * Returns the performance on this observation.
	 * 
	 * @return the performance on this observation.
	 */
	public double getValue()
	{
		return value;
	}

	/**
	 * Sets the performance on this observation.
	 * 
	 * @param value the value to set.
	 */
	public void setValue(double value)
	{
		this.value = value;
	}
}