/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer;

import java.util.List;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.framework.NumericCost;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class FmeasureEvaluator
	extends RuleEvaluator
{
	public FmeasureEvaluator(Dataset dataset)
	{
		super(dataset);
	}

	@Override
	public NumericCost evaluate(HybridRule rule)
	{
		String last = rule.getConsequent().getLast().getLabel();
		List<Instance> available = dataset.getInstances(last);

		Rule antecedent = rule.getAntecedent();
		int correct = 0;

		for (Instance instance : available)
		{
			if (antecedent.isCovered(instance))
			{
				correct++;
			}
		}

		int covered = Evaluator.findCoveredCases(antecedent, dataset).size();
		double precision = correct / (double) covered;
		double recall = correct / (double) available.size();
		// assuming B = 1
		double fmeasure = (2 * precision * recall) / (precision + recall);

		return new NumericCost(!isNumber(fmeasure) ? 0.0 : fmeasure);
	}
}