/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer;

import net.sf.myra.framework.Cost;
import net.sf.myra.framework.NumericCost;
import net.sf.myra.datamining.data.Dataset;

/**
 * This class represents a rule evaluator which combines a hierarchical
 * measure (based on <code>hFmeasureEvaluator</code> class) and a local
 * measure (based on <code>FmeasureEvaluator</code> class).
 * 
 * @see hFmeasureEvaluator
 * @see FmeasureEvaluator
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class HybridEvaluator
	extends RuleEvaluator
{
	/**
	 * The 'hierarchical' rule evaluator (Fmeasure).
	 */
	private hFmeasureEvaluator hEvaluator;

	/**
	 * The 'local' rule evaluator (PrecisionRecall).
	 */
	private FmeasureEvaluator lEvaluator;

	/**
	 * Default constructor.
	 * 
	 * @param dataset the dataset instance.
	 */
	public HybridEvaluator(Dataset dataset)
	{
		super(dataset);
		this.hEvaluator = new hFmeasureEvaluator(dataset);
		this.lEvaluator = new FmeasureEvaluator(dataset);
	}

	@Override
	public Cost evaluate(HybridRule rule)
	{
		double c1 = hEvaluator.evaluate(rule).getValue();
		double c2 = lEvaluator.evaluate(rule).getValue();

		double average = (2 * c1 * c2) / (c1 + c2);

		return new NumericCost(!isNumber(average) ? 0.0 : average);
	}
}