/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer;

import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.framework.Cost;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public abstract class RuleEvaluator
{
	/**
	 * The dataset reference.
	 */
	protected Dataset dataset;

	/**
	 * Default constructor.
	 * 
	 * @param dataset
	 */
	public RuleEvaluator(Dataset dataset)
	{
		this.dataset = dataset;
	}

	/**
	 * Tests if the specified value is a number.
	 * 
	 * @param value the value to be tested.
	 * 
	 * @return <code>true</code> if the value is a number; <code>false</code>
	 *         otherwise.
	 */
	protected boolean isNumber(double value)
	{
		return !Double.isNaN(value) && !Double.isInfinite(value);
	}

	/**
	 * Returns the cost associated with the evaluation of the specified
	 * hybrid rule.
	 * 
	 * @param rule the hybrid rule to be evaluated.
	 * 
	 * @return the cost associated with the evaluation of the specified
	 * hybrid rule.
	 */
	public abstract Cost evaluate(HybridRule rule);
}