/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer;

import net.sf.myra.framework.Ant;
import net.sf.myra.framework.Colony;
import net.sf.myra.framework.HeuristicInformation;
import net.sf.myra.framework.PheromoneUpdater;
import net.sf.myra.framework.Trail;
import net.sf.myra.framework.TrailFactory;

/**
 * This class represents an ant colony
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class Species
{
	/**
	 * The colony of ants representing this species.
	 */
	private Colony colony;

	/**
	 * The pheromone update policy.
	 */
	private PheromoneUpdater updater;

	/**
	 * The heuristic information.
	 */
	private HeuristicInformation information;

	/**
	 * Default constructor.
	 * 
	 * @param updater the pheromone updater instance.
	 * @param factory the trail factory instance.
	 */
	public Species(PheromoneUpdater updater, HeuristicInformation information,
			TrailFactory factory)
	{
		this.updater = updater;
		this.information = information;

		colony = new Colony();
		colony.setTrailFactory(factory);
	}

	/**
	 * Initializes the colony.
	 */
	public void initialize()
	{
		updater.initialize(colony);
		information.initialize(colony.getTrailFactory().getGraph());
	}

	/**
	 * Creates ant trails.
	 */
	public void iterate()
	{
		for (Ant ant : colony.getPopulation())
		{
			ant.walk();
		}
	}

	/**
	 * Updates the pheromone trail represented by the specified trail.
	 * 
	 * @param trail the trail used to update the pheromone.
	 */
	public void update(Trail trail)
	{
		updater.update(trail, colony);
	}

	/**
	 * Returns the colony instance associated.
	 * 
	 * @return the colony instance associated.
	 */
	public Colony getColony()
	{
		return colony;
	}
}