/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer;

import java.util.List;
import net.sf.myra.datamining.Evaluator;
import net.sf.myra.datamining.Rule;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.framework.NumericCost;

/**
 * This class represents the hierarchical F-measure evaluator.
 * 
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class hFmeasureEvaluator
	extends RuleEvaluator
{
	/**
	 * Default constructor.
	 * 
	 * @param dataset
	 */
	public hFmeasureEvaluator(Dataset dataset)
	{
		super(dataset);
	}

	@Override
	public NumericCost evaluate(HybridRule rule)
	{
		Metadata metadata = dataset.getMetadata();
		Rule antecedent = rule.getAntecedent();

		Label pLabel = metadata.expand(antecedent.getConsequent());

		if (!antecedent.getConsequent().equals(pLabel))
		{
			antecedent = antecedent.clone();
			antecedent.setConsequent(pLabel);
		}

		List<Instance> instances =
			Evaluator.findCoveredCases(antecedent, dataset);

		int correct = 0;
		int predicted = 0;
		int total = 0;

		for (Instance instance : instances)
		{
			Label tLabel = instance.getLabel();

			// the root label is not considered as a hit
			correct += (tLabel.intersect(pLabel) - 1);
			predicted += (pLabel.cardinality() - 1);
			total += (tLabel.cardinality() - 1);
		}
		
		double hP = correct / (double) predicted;
		double hR = correct / (double) total;
		double hF = (2 * hP * hR) / (hP + hR);

		return new NumericCost(isNumber(hF) ? hF : 0.0);
	}
}