/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer.multilabel;

import java.util.HashMap;

import net.sf.myra.cantminer.cAntMiner2;
import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.NominalAttribute;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.HeuristicInformation;
import net.sf.myra.framework.HeuristicValue;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ClassAwareHeuristicInformation
	implements HeuristicInformation
{
	/**
	 * The (current) consequent class value of the rules.
	 */
	private String value;

	/**
	 * The current data set.
	 */
	private Dataset dataset;

	/**
	 * Default constructor.
	 * 
	 * @param dataset
	 * @param value
	 */
	public ClassAwareHeuristicInformation(Dataset dataset, String value)
	{
		this.dataset = dataset;
		this.value = value;
	}

	@SuppressWarnings("unchecked")
	public void initialize(Graph<?> graph)
	{
		HashMap<String, int[]> frequencies = new HashMap<String, int[]>();

		Metadata metadata = dataset.getMetadata();
		int cIndex = metadata.getLabel().getIndex(value);

		for (Attribute attribute : metadata.getPredictor())
		{
			if (attribute instanceof NominalAttribute)
			{
				NominalAttribute nominal = (NominalAttribute) attribute;
				int aIndex = metadata.getIndex(attribute.getName());

				for (String value : nominal.getValues())
				{
					int vIndex = nominal.getIndex(value);
					int[] counter = new int[2];

					for (Instance instance : dataset)
					{
						if (instance.values()[aIndex] == vIndex)
						{
							counter[instance.labels()[cIndex] ? 0 : 1]++;
						}
					}

					frequencies.put(nominal.toString(value), counter);
				}
			}
		}

		double log_k =
			Math.log(((NominalAttribute) metadata.getTarget()).getValues()
				.size())
				/ Math.log(2.0);

		for (Vertex<?> vertex : graph.getVertices())
		{
			if (!cAntMiner2.VIRTUAL_VERTEX.equals(vertex.getLabel()))
			{
				int[] counter = frequencies.get(vertex.getLabel());
				double value = 0.0;

				// is this vertex a continuous attribute condition?
				if (counter == null)
				{
					ContinuousAttribute attribute =
						(ContinuousAttribute) metadata
							.get(((Vertex<Term>) vertex).getInfo()
								.getAttribute());

					value = log_k - attribute.entropy(dataset.getInstances());
				}
				// for nominal attribute conditions, use the (current) class
				// frequency
				else
				{
					double total = counter[0] + counter[1];
					value = (total > 0 ? log_k * (counter[0] / total) : 0.0);
				}

				vertex.setHeuristicValue(new HeuristicValue(value));
			}
		}
	}
}