/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008-2009  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer.multilabel;

import java.util.List;
import net.sf.myra.cantminer.EdgeTrailFactory;
import net.sf.myra.cantminer.ThresholdHelper;
import net.sf.myra.datamining.data.Attribute;
import net.sf.myra.datamining.data.ContinuousAttribute;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Metadata;
import net.sf.myra.datamining.data.Term;
import net.sf.myra.framework.Graph;
import net.sf.myra.framework.Vertex;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class ClassAwareTrailFactory
	extends EdgeTrailFactory
{
	/**
	 * The (current) consequent class value of the rules.
	 */
	private String value;

	/**
	 * The threshold selection helper.
	 */
	private ThresholdHelper helper;

	/**
	 * Default constructor.
	 * 
	 * @param graph the problem construction graph.
	 * @param dataset the dataset reference.
	 * @param value the consequent class value of the rules.
	 */
	public ClassAwareTrailFactory(Graph<Term> graph, Dataset dataset,
			String value)
	{
		super(graph, dataset);
		this.value = value;
		this.helper = new ThresholdHelper(dataset.getMetadata());
	}

	@Override
	protected Vertex<Term> process(Metadata metadata, Vertex<Term> vertex,
			List<Instance> covered, boolean first)
	{
		Attribute attribute = metadata.get(vertex.getInfo().getIndex());

		if (attribute instanceof ContinuousAttribute)
		{
			return helper.create((ContinuousAttribute) attribute, vertex,
				covered, value);
		}

		return vertex;
	}
}