
package net.sf.myra.hantminer.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

import net.sf.myra.datamining.statistics.CurveFactory;
import net.sf.myra.datamining.statistics.Information;
import net.sf.myra.datamining.statistics.Parser;
import net.sf.myra.datamining.statistics.Point;

public class ClusThreshold
{
	//private static final String CLUS_FOLDER = "results";

	private static final String HMANTMINER_FOLDER = "results-hmantminer";

	private static final String FOLD_PREFIX = "FOLD-";

	private static final String HP_PREFIX = "avg_hP,";

	private static final String HR_PREFIX = "avg_hR,";

	private static final String HF_PREFIX = "avg_hF,";

	public static void main(String[] args)
		throws Exception
	{
		File results = new File(args[0]);
		BufferedReader reader = new BufferedReader(new FileReader(results));
		String line = null;

		Map<String, ArrayList<Result>> mapping =
			new LinkedHashMap<String, ArrayList<Result>>();
		ArrayList<Result> current = null;
		String dataset = null;

		while ((line = reader.readLine()) != null)
		{
			if (line.startsWith(FOLD_PREFIX))
			{
				Result result = new Result();

				int start = line.indexOf(HP_PREFIX) + HP_PREFIX.length();
				result.hP =
					Double.parseDouble(line.substring(start,
						line.indexOf(',', start)));

				start = line.indexOf(HR_PREFIX) + HR_PREFIX.length();
				result.hR =
					Double.parseDouble(line.substring(start,
						line.indexOf(',', start)));

				start = line.indexOf(HF_PREFIX) + HF_PREFIX.length();
				result.hF =
					Double.parseDouble(line.substring(start,
						line.indexOf(',', start)));

				current.add(result);
			}
			else
			{
				line = clean(line);

				if (line.length() > 0)
				{
					current = new ArrayList<Result>(10);
					dataset = line;
				}
				else
				{
					if (current != null)
					{
						mapping.put(dataset, current);
					}

					current = null;
					dataset = null;
				}
			}
		}

		reader.close();

		// parses Clus output files
		/*
				for (String name : mapping.keySet())
				{
					File directory =
						new File(new File(results.getParent(), CLUS_FOLDER), "D-"
							+ name);

					System.out.println(directory.getName());

					for (File output : directory.listFiles())
					{
						int index =
							Integer.parseInt(output.getName().substring(0,
								output.getName().indexOf('_')));

						System.out.print(index);

						current = mapping.get(name);
						Result result = current.get(index);

						parse(output, result.hR, result.hP);
					}

					System.out.println();
				}
		*/
		for (String name : mapping.keySet())
		{
			File directory =
				new File(new File(results.getParent(), HMANTMINER_FOLDER), name);

			if (directory.exists())
			{
				System.out.println(directory.getName());

				double[][] precision = new double[10][];
				double[][] recall = new double[10][];

				for (int i = 0; i < precision.length; i++)
				{
					precision[i] = new double[10];
					recall[i] = new double[10];
				}

				for (File output : directory.listFiles())
				{
					if (output.getName().endsWith(".pred"))
					{
						int index =
							Integer.parseInt(output.getName().substring(
								output.getName().lastIndexOf('-') + 1,
								output.getName().lastIndexOf('.')));

						int run =
							Integer.parseInt(output.getName().substring(0,
								output.getName().indexOf('-')));

						//System.out.print(index);

						current = mapping.get(name);
						Result result = current.get(index);

						double[] values = parse(output, result.hR, result.hP);
						precision[index][run - 1] = values[0];
						recall[index][run - 1] = values[1];
					}
				}

				for (int i = 0; i < precision.length; i++)
				{
					System.out.print(i);
					System.out.print(",");
					System.out.print(stdev(precision[i]));
					System.out.print(",");
					System.out.println(stdev(recall[i]));
				}

				System.out.println();
			}
		}
	}

	public static double[] parse(File output, double rcThreshold,
		double prThreshold)
		throws Exception
	{
		Parser parser = new Parser();
		Information information =
			parser.read(output, CurveFactory.PRECISION_RECALL);

		Point previous = null;
		Point highest = null;
		double precision = Double.NaN;

		for (Point point : information.getCurve().getPoints())
		{
			// precision
			if (Double.isNaN(precision) && point.getX() >= rcThreshold)
			{
				if (point.getX() == rcThreshold || previous == null)
				{
					precision = point.getY();
				}
				else
				{
					precision = (previous.getY() + point.getY()) / 2.0;
				}
			}

			// recall
			if (point.getY() >= prThreshold)
			{
				if (highest == null || highest.getX() < point.getX())
				{
					highest = point;
				}
			}

			previous = point;
		}

		//System.out.print("," + precision);
		//System.out.println("," + highest.getX());
		return new double[] { precision, highest.getX() };
	}

	private static String clean(String s)
	{
		String clean =
			s.replaceAll("(\\.|,)+", " ").replaceAll("(\'|\")+", " ").trim();
		int index = clean.indexOf('|');

		// removes the text after the comment '|' character
		if (index != -1)
		{
			clean = clean.substring(0, index).trim();
		}

		return clean;
	}

	public static String stdev(double[] numbers)
	{
		if (numbers.length == 0)
		{
			return "0.0 +/- 0.0";
		}

		double mean = 0;
		double total = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			if (numbers[i] > 0)
			{
				mean += numbers[i];
				total++;
			}
		}

		mean /= total;
		double std = 0;

		for (int i = 0; i < numbers.length; i++)
		{
			if (numbers[i] > 0)
			{
				std += Math.pow(numbers[i] - mean, 2);
			}
		}

		std /= (total - 1);
		std /= total;
		std = Math.sqrt(std);

		if (Double.isNaN(std) || Double.isInfinite(std))
		{
			std = 0.0;
		}

		NumberFormat formatter = new DecimalFormat("#,##0.000");
		return formatter.format(mean) + " +/- " + formatter.format(std);
	}

	private static class Result
	{
		double hR;
		double hP;

		@SuppressWarnings("unused")
		double hF;
	}
}