/*
 * $Id$
 * 
 * ============================================================================
 *
 * This file is part of Myra.
 *
 * Myra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Myra is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Myra. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright (C) 2008  Fernando Esteban Barril Otero
 * 
 * ============================================================================
 */

package net.sf.myra.hantminer.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import net.sf.myra.cantminer.cAntMiner;
import net.sf.myra.datamining.Classifier;
import net.sf.myra.datamining.Gatherer;
import net.sf.myra.datamining.Model;
import net.sf.myra.datamining.data.ClassHierarchy;
import net.sf.myra.datamining.data.Dataset;
import net.sf.myra.datamining.data.Instance;
import net.sf.myra.datamining.data.Label;
import net.sf.myra.datamining.data.Node;
import net.sf.myra.datamining.data.Strategy;
import net.sf.myra.datamining.model.AbstractRuleModel;
import net.sf.myra.framework.Configuration;

/**
 * @author Fernando Esteban Barril Otero
 * @version $Revision$ $Date::                     $
 */
public class cAntMinerTD
	extends Classifier
{
	/**
	 * Default constructor.
	 */
	public cAntMinerTD()
	{
		super(null);
	}

	public TopDownRuleModel run(Dataset dataset)
	{
		ClassHierarchy hierarchy = dataset.getMetadata().getClassHierarchy();
		TopDownRuleModel model = new TopDownRuleModel(hierarchy);

		Strategy.Type strategy =
			Strategy.get(Configuration.getProperty(Strategy.TYPE_KEY,
				Strategy.DEFAULT_TYPE));

		for (Node node : dataset.getMetadata().getClassHierarchy().flat())
		{
			if (!node.isRoot())
			{
				try
				{
					Dataset local = strategy.transform(node, dataset);
					cAntMiner classifier = new cAntMiner();
					AbstractRuleModel rules = classifier.run(local);

					model.set(node.getLabel(), rules);
				}
				catch (Exception e)
				{
					throw new RuntimeException(e);
				}
			}
		}

		return model;
	}

	/**
	 * This class represents the hierarchical decision tree model.
	 */
	private class TopDownRuleModel
		implements Model
	{
		/**
		 * The mapping of (class label, decision tree) instances.
		 */
		private Map<String, AbstractRuleModel> mapping;

		/**
		 * The class hierarchy.
		 */
		private ClassHierarchy hierarchy;

		/**
		 * Default constructor.
		 * 
		 * @param hierarchy the class hierarchy.
		 */
		public TopDownRuleModel(ClassHierarchy hierarchy)
		{
			this.hierarchy = hierarchy;
			mapping =
				new HashMap<String, AbstractRuleModel>(hierarchy.getSize() - 1,
					1.0f);
		}

		/**
		 * Sets the model for the specified class label.
		 * 
		 * @param label the class label.
		 * @param model the model to set.
		 */
		public void set(String label, AbstractRuleModel model)
		{
			mapping.put(label, model);
		}
		
		public Gatherer getGatherer()
		{
			throw new UnsupportedOperationException();
		}

		/**
		 * Returns the predicted label for the specified instance. The instance
		 * is classified in a top-down fashion.
		 * 
		 * @param instance the instance to predict the class label.
		 * 
		 * @return the predicted label for the specified instance.
		 */
		public Label classify(Instance instance)
		{
			HashSet<String> predicted = new HashSet<String>();
			LinkedList<String> candidates = new LinkedList<String>();

			for (Node node : hierarchy.getRoot().getChildren())
			{
				candidates.add(node.getLabel());
			}

			predicted.add(hierarchy.getRoot().getLabel());

			while (!candidates.isEmpty())
			{
				String current = candidates.removeFirst();
				AbstractRuleModel model = mapping.get(current);

				if (Strategy.POSITIVE.equals(model.classify(instance)
					.toString()))
				{
					predicted.add(current);

					for (Node node : hierarchy.find(current).getChildren())
					{
						candidates.add(node.getLabel());
					}
				}
			}

			return instance.getLabel().toLabel(predicted);
		}

		@Override
		public String toString()
		{
			return "(TopDown Rule Model)\n";
		}
	}
}